# AUTOGENERATED! DO NOT EDIT! File to edit: ../00_ConvertToFeems.ipynb.

# %% auto 0
__all__ = [
    "convert_proto_point_to_list",
    "convert_proto_curve1d_to_np_array",
    "convert_proto_efficiency_bsfc_power_to_np_array",
    "convert_proto_electric_component_to_feems",
    "convert_proto_electric_machine_to_feems",
    "convert_proto_fuel_cell_system_to_feems",
    "convert_emission_curve_to_feems",
    "convert_nox_calculation_method",
    "convert_proto_engine_to_feems",
    "convert_proto_cogas_to_feems",
    "convert_proto_genset_to_feems",
    "convert_proto_coges_to_feems",
    "convert_proto_battery_to_feems",
    "convert_proto_battery_system_to_feems",
    "convert_proto_supercapacitor_to_feems",
    "convert_proto_supercapacitor_system_to_feems",
    "collect_electric_components_from_sub_system",
    "convert_proto_generic_electric_component_to_feems",
    "get_component_type",
    "convert_proto_pti_pto_subsystem_to_feems",
    "convert_proto_serial_subsystem_to_feems",
    "convert_generic_electric_subsystem_to_feems",
    "convert_proto_switchboard_to_feems",
    "convert_proto_shaftline_to_feems",
    "convert_feems_switchboards_to_feems_electric_power_system",
    "convert_feems_shaftlines_to_feems_mechanical_propulsion_system",
    "convert_proto_electric_system_to_feems",
    "convert_proto_mechanical_system_to_feems",
    "convert_proto_propulsion_system_to_feems",
]

# %% ../00_ConvertToFeems.ipynb 3
from functools import reduce
from typing import Union, List, Dict, Any, Optional

import numpy as np
from feems.components_model import (
    Engine,
    Switchboard,
    ShaftLine,
    BasicComponent,
    MechanicalPropulsionComponent,
)
from feems.components_model.component_electric import (
    ElectricComponent,
    FuelCellSystem,
    ElectricMachine,
    BatterySystem,
    Battery,
    SuperCapacitorSystem,
    PTIPTO,
    SuperCapacitor,
    FuelCell,
    COGES,
)
from feems.components_model.component_electric import SerialSystemElectric, Genset
from feems.components_model.component_mechanical import COGAS, EngineDualFuel
from feems.fuel import FuelOrigin, TypeFuel
from feems.system_model import (
    ElectricPowerSystem,
    MechanicalPropulsionSystem,
    HybridPropulsionSystem,
    MechanicalPropulsionSystemWithElectricPowerSystem,
    MainEngineForMechanicalPropulsion,
    MainEngineWithGearBoxForMechanicalPropulsion,
)
from feems.types_for_feems import (
    TypeComponent,
    TypePower,
    EmissionCurve,
    EmissionCurvePoint,
    EmissionType,
    NOxCalculationMethod,
)

import MachSysS.system_structure_pb2 as proto


def convert_proto_point_to_list(point: proto.Point) -> List[float]:
    """Converts protobuf point to a list"""
    return [point.x, point.y]


def convert_proto_curve1d_to_np_array(curve: proto.Curve1D) -> np.ndarray:
    """Converts protobuf curve1d to numpy array"""
    return np.array(list(map(convert_proto_point_to_list, curve.points)))


def convert_proto_efficiency_bsfc_power_to_np_array(
    efficiency_bsfc_power: Union[proto.Efficiency, proto.BSFC, proto.PowerCurve]
) -> np.ndarray:
    """Converts protobuf efficiency or bsfc to numpy array"""
    if isinstance(efficiency_bsfc_power, proto.PowerCurve):
        return convert_proto_curve1d_to_np_array(efficiency_bsfc_power.curve)
    if efficiency_bsfc_power.HasField("value"):
        if efficiency_bsfc_power.value > 0:
            return np.array([efficiency_bsfc_power.value])
    if efficiency_bsfc_power.HasField("curve"):
        return convert_proto_curve1d_to_np_array(efficiency_bsfc_power.curve.curve)
    else:
        raise TypeError("The efficiency value or curve is not properly set.")


def convert_proto_electric_component_to_feems(
    proto_component: proto.ElectricComponent,
    component_type: TypeComponent,
    power_type: TypePower,
    switchboard_id: int = 0,
) -> ElectricComponent:
    return ElectricComponent(
        type_=component_type,
        name=proto_component.name,
        rated_power=proto_component.rated_power_kw,
        eff_curve=convert_proto_efficiency_bsfc_power_to_np_array(
            proto_component.efficiency
        ),
        power_type=power_type,
        switchboard_id=switchboard_id,
    )


def convert_proto_electric_machine_to_feems(
    proto_component: proto.ElectricMachine,
    component_type: TypeComponent,
    power_type: TypePower,
    switchboard_id: int = 0,
) -> ElectricMachine:
    return ElectricMachine(
        type_=component_type,
        name=proto_component.name,
        rated_power=proto_component.rated_power_kw,
        rated_speed=proto_component.rated_speed_rpm,
        power_type=power_type,
        eff_curve=convert_proto_efficiency_bsfc_power_to_np_array(
            proto_component.efficiency
        ),
        switchboard_id=switchboard_id,
    )


def convert_proto_fuel_cell_system_to_feems(
    subsystem: proto.Subsystem, switchboard_id: int
) -> FuelCellSystem:
    """Converts protobuf subsystem message to feems component"""
    number_modules = 1
    if subsystem.fuel_cell.number_modules > 1:
        number_modules = subsystem.fuel_cell.number_modules
    fuel_cell = FuelCell(
        name=subsystem.fuel_cell.name,
        rated_power=subsystem.fuel_cell.rated_power_kw,
        eff_curve=convert_proto_efficiency_bsfc_power_to_np_array(
            subsystem.fuel_cell.efficiency
        ),
        fuel_type=TypeFuel(subsystem.fuel_cell.fuel.fuel_type),
        fuel_origin=FuelOrigin(subsystem.fuel_cell.fuel.fuel_origin),
    )
    converter = convert_proto_electric_component_to_feems(
        subsystem.converter1,
        component_type=TypeComponent.POWER_CONVERTER,
        power_type=TypePower.POWER_TRANSMISSION,
        switchboard_id=switchboard_id,
    )
    return FuelCellSystem(
        name=subsystem.name,
        fuel_cell_module=fuel_cell,
        converter=converter,
        switchboard_id=switchboard_id,
        number_modules=number_modules,
    )


def convert_emission_curve_to_feems(
    emission_curve: proto.EmissionCurve,
) -> EmissionCurve:
    """Converts protobuf emission curve to numpy array"""
    points_per_kwh = [
        EmissionCurvePoint(load_ratio=point.x, emission_g_per_kwh=point.y)
        for point in emission_curve.curve.points
    ]
    return EmissionCurve(
        points_per_kwh=points_per_kwh,
        emission=EmissionType(emission_curve.emission_type),
    )


def convert_nox_calculation_method(
    proto_comp: Union[proto.Engine, proto.COGAS]
) -> NOxCalculationMethod:
    """Converts protobuf nox calculation type to feems nox calculation method"""
    if isinstance(proto_comp, proto.Engine):
        nox_calculation_method = NOxCalculationMethod.TIER_2
    elif isinstance(proto_comp, proto.COGAS):
        nox_calculation_method = NOxCalculationMethod.TIER_3
    else:
        raise TypeError("The component should be either an engine or COGAS")
    if proto_comp.nox_calculation_method is not None:
        name = proto.Engine.NOxCalculationMethod.Name(proto_comp.nox_calculation_method)
        nox_calculation_method = NOxCalculationMethod[name]
    return nox_calculation_method


def convert_proto_engine_to_feems(
    proto_engine: proto.Engine,
    type_engine: TypeComponent = TypeComponent.AUXILIARY_ENGINE,
) -> Engine:
    """Converts protobuf engine message to feems engine component"""
    nox_calculation_method = convert_nox_calculation_method(proto_engine)
    emission_curves = (
        [
            convert_emission_curve_to_feems(emission_curve)
            for emission_curve in proto_engine.emission_curves
        ]
        if proto_engine.emission_curves
        else None
    )
    if proto_engine.HasField("pilot_bsfc"):
        return EngineDualFuel(
            type_=type_engine,
            name=proto_engine.name,
            rated_power=proto_engine.rated_power_kw,
            rated_speed=proto_engine.rated_speed_rpm,
            bsfc_curve=convert_proto_efficiency_bsfc_power_to_np_array(
                proto_engine.bsfc
            ),
            bspfc_curve=convert_proto_efficiency_bsfc_power_to_np_array(
                proto_engine.pilot_bsfc
            ),
            fuel_type=TypeFuel(proto_engine.main_fuel.fuel_type),
            fuel_origin=FuelOrigin(proto_engine.main_fuel.fuel_origin),
            pilot_fuel_type=TypeFuel(proto_engine.pilot_fuel.fuel_type),
            pilot_fuel_origin=FuelOrigin(proto_engine.pilot_fuel.fuel_origin),
            nox_calculation_method=nox_calculation_method,
            emissions_curves=emission_curves,
        )
    return Engine(
        type_=type_engine,
        name=proto_engine.name,
        rated_power=proto_engine.rated_power_kw,
        rated_speed=proto_engine.rated_speed_rpm,
        bsfc_curve=convert_proto_efficiency_bsfc_power_to_np_array(proto_engine.bsfc),
        fuel_type=TypeFuel(proto_engine.main_fuel.fuel_type),
        fuel_origin=FuelOrigin(proto_engine.main_fuel.fuel_origin),
        nox_calculation_method=nox_calculation_method,
        emissions_curves=emission_curves,
    )


def convert_proto_cogas_to_feems(
    proto_cogas: proto.COGAS,
) -> Engine:
    """Converts protobuf COGAS message to feems COGAS component"""
    nox_calculation_method = convert_nox_calculation_method(proto_cogas)
    emission_curves = (
        [
            convert_emission_curve_to_feems(emission_curve)
            for emission_curve in proto_cogas.emission_curves
        ]
        if proto_cogas.emission_curves
        else None
    )
    return COGAS(
        name=proto_cogas.name,
        rated_power=proto_cogas.rated_power_kw,
        rated_speed=proto_cogas.rated_speed_rpm,
        eff_curve=convert_proto_efficiency_bsfc_power_to_np_array(
            proto_cogas.efficiency
        ),
        gas_turbine_power_curve=convert_proto_efficiency_bsfc_power_to_np_array(
            proto_cogas.gas_turbine_power_curve
        ),
        steam_turbine_power_curve=convert_proto_efficiency_bsfc_power_to_np_array(
            proto_cogas.steam_turbine_power_curve
        ),
        fuel_type=TypeFuel(proto_cogas.fuel.fuel_type),
        fuel_origin=FuelOrigin(proto_cogas.fuel.fuel_origin),
        nox_calculation_method=nox_calculation_method,
        emissions_curves=emission_curves,
    )


def convert_proto_genset_to_feems(
    subsystem: proto.Subsystem, switchboard_id: int
) -> Genset:
    """Converts protobuf subsystem message to feems component"""
    engine = convert_proto_engine_to_feems(proto_engine=subsystem.engine)
    generator = convert_proto_electric_machine_to_feems(
        proto_component=subsystem.electric_machine,
        component_type=TypeComponent.SYNCHRONOUS_MACHINE,
        power_type=TypePower.POWER_SOURCE,
        switchboard_id=switchboard_id,
    )
    rectifier = None
    if subsystem.HasField("converter1"):
        rectifier = convert_proto_electric_component_to_feems(
            proto_component=subsystem.converter1,
            component_type=TypeComponent.RECTIFIER,
            power_type=TypePower.POWER_TRANSMISSION,
            switchboard_id=switchboard_id,
        )
    return Genset(
        name=subsystem.name, aux_engine=engine, generator=generator, rectifier=rectifier
    )


def convert_proto_coges_to_feems(
    subsystem: proto.Subsystem, switchboard_id: int
) -> COGES:
    """Converts protobuf subsystem message to feems component"""
    cogas = convert_proto_cogas_to_feems(proto_cogas=subsystem.cogas)
    generator = convert_proto_electric_machine_to_feems(
        proto_component=subsystem.electric_machine,
        component_type=TypeComponent.SYNCHRONOUS_MACHINE,
        power_type=TypePower.POWER_SOURCE,
        switchboard_id=switchboard_id,
    )
    return COGES(name=subsystem.name, cogas=cogas, generator=generator)


def convert_proto_battery_to_feems(
    proto_component: proto.Battery, switchboard_id: int = 0
) -> Battery:
    return Battery(
        name=proto_component.name,
        rated_capacity_kwh=proto_component.energy_capacity_kwh,
        charging_rate_c=proto_component.rated_charging_rate_c,
        discharge_rate_c=proto_component.rated_discharging_rate_c,
        eff_charging=proto_component.efficiency_charging,
        eff_discharging=proto_component.efficiency_discharging,
        soc0=proto_component.initial_state_of_charge,
        switchboard_id=switchboard_id,
    )


def convert_proto_battery_system_to_feems(
    subsystem: proto.Subsystem, switchboard_id: int
) -> BatterySystem:
    """Converts protobuf subsystem message to feems component"""
    battery = convert_proto_battery_to_feems(
        proto_component=subsystem.battery,
    )
    converter = convert_proto_electric_component_to_feems(
        proto_component=subsystem.converter1,
        component_type=TypeComponent.POWER_CONVERTER,
        power_type=TypePower.POWER_TRANSMISSION,
    )
    return BatterySystem(
        name=subsystem.name,
        battery=battery,
        converter=converter,
        switchboard_id=switchboard_id,
    )


def convert_proto_supercapacitor_to_feems(
    proto_component: proto.SuperCapacitor, switchboard_id: int = 0
) -> SuperCapacitor:
    return SuperCapacitor(
        name=proto_component.name,
        rated_capacity_wh=proto_component.energy_capacity_wh,
        rated_power=proto_component.rated_power_kw,
        eff_charging=proto_component.efficiency_charging,
        eff_discharging=proto_component.efficiency_discharging,
        soc0=proto_component.initial_state_of_charge,
        switchboard_id=switchboard_id,
    )


def convert_proto_supercapacitor_system_to_feems(
    subsystem: proto.Subsystem, switchboard_id: int
) -> SuperCapacitorSystem:
    """Converts protobuf subsystem message to feems component"""
    supercapacitor = convert_proto_supercapacitor_to_feems(
        proto_component=subsystem.battery,
    )
    converter = convert_proto_electric_component_to_feems(
        proto_component=subsystem.converter1,
        component_type=TypeComponent.POWER_CONVERTER,
        power_type=TypePower.POWER_TRANSMISSION,
    )
    return SuperCapacitorSystem(
        name=subsystem.name,
        supercapacitor=supercapacitor,
        converter=converter,
        switchboard_id=switchboard_id,
    )


def collect_electric_components_from_sub_system(
    subsystem: proto.Subsystem,
) -> List[Dict[str, Union[str, Union[proto.ElectricMachine, proto.ElectricComponent]]]]:
    field_names = [
        "electric_machine",
        "transformer",
        "converter1",
        "converter2",
        "propeller",
        "other_load",
    ]
    components = [
        {"name": field_name, "proto_component": getattr(subsystem, field_name)}
        for field_name in field_names
        if subsystem.HasField(field_name)
    ]
    return sorted(
        components,
        key=lambda component: component.get(
            "proto_component"
        ).order_from_switchboard_or_shaftline,
    )


def convert_proto_generic_electric_component_to_feems(
    proto_component: Union[proto.ElectricComponent, proto.ElectricMachine],
    component_type: TypeComponent,
    power_type: TypePower,
    switchboard_id: int = 0,
) -> Union[ElectricComponent, ElectricMachine]:
    if proto_component.DESCRIPTOR.name == "ElectricComponent":
        return convert_proto_electric_component_to_feems(
            proto_component=proto_component,
            component_type=component_type,
            power_type=power_type,
            switchboard_id=switchboard_id,
        )
    else:
        return convert_proto_electric_machine_to_feems(
            proto_component=proto_component,
            component_type=component_type,
            power_type=power_type,
            switchboard_id=switchboard_id,
        )


def get_component_type(component_category: str) -> TypeComponent:
    if component_category == "transformer":
        return TypeComponent.TRANSFORMER
    if component_category in ["converter1", "converter2"]:
        return TypeComponent.POWER_CONVERTER
    if component_category == "electric_machine":
        return TypeComponent.SYNCHRONOUS_MACHINE
    if component_category == "other_load":
        return TypeComponent.OTHER_LOAD
    else:
        raise TypeError("The component category cannot be determined.")


def convert_proto_pti_pto_subsystem_to_feems(
    subsystem: proto.Subsystem,
    switchboard_id: int,
    shaft_line_id: int = 1,
) -> PTIPTO:
    proto_components = collect_electric_components_from_sub_system(subsystem)
    components_feems = [
        convert_proto_generic_electric_component_to_feems(
            proto_component=each_proto_component.get("proto_component"),
            component_type=get_component_type(each_proto_component.get("name")),
            power_type=TypePower.PTI_PTO,
        )
        for each_proto_component in proto_components
    ]
    return PTIPTO(
        name=subsystem.name,
        components=components_feems,
        rated_power=None if subsystem.rated_power_kw == 0 else subsystem.rated_power_kw,
        rated_speed=subsystem.rated_speed_rpm,
        switchboard_id=switchboard_id,
        shaft_line_id=shaft_line_id,
    )


def convert_proto_serial_subsystem_to_feems(
    subsystem: proto.Subsystem, switchboard_id: int
) -> SerialSystemElectric:
    proto_components = collect_electric_components_from_sub_system(subsystem)
    components_feems = [
        convert_proto_generic_electric_component_to_feems(
            proto_component=each_proto_component.get("proto_component"),
            component_type=get_component_type(each_proto_component.get("name")),
            power_type=TypePower(subsystem.power_type),
        )
        for each_proto_component in proto_components
    ]
    return SerialSystemElectric(
        type_=TypeComponent(subsystem.component_type),
        name=subsystem.name,
        power_type=TypePower(subsystem.power_type),
        components=components_feems,
        switchboard_id=switchboard_id,
        rated_power=None if subsystem.rated_power_kw == 0 else subsystem.rated_power_kw,
        rated_speed=(
            None if subsystem.rated_speed_rpm == 0 else subsystem.rated_speed_rpm
        ),
    )


def convert_generic_electric_subsystem_to_feems(
    subsystem: proto.Subsystem, switchboard_id: int
) -> Union[ElectricComponent, SerialSystemElectric, PTIPTO]:
    proto_components = collect_electric_components_from_sub_system(subsystem)
    if len(proto_components) == 1:
        component = proto_components[0].get("proto_component")
        if component.name == "":
            component.name = subsystem.name
        return convert_proto_generic_electric_component_to_feems(
            proto_component=component,
            component_type=TypeComponent(subsystem.component_type),
            power_type=TypePower(subsystem.power_type),
            switchboard_id=switchboard_id,
        )
    else:
        if subsystem.component_type == proto.Subsystem.ComponentType.PTI_PTO_SYSTEM:
            return convert_proto_pti_pto_subsystem_to_feems(
                subsystem=subsystem,
                switchboard_id=switchboard_id,
            )
        else:
            return convert_proto_serial_subsystem_to_feems(
                subsystem=subsystem, switchboard_id=switchboard_id
            )


def convert_proto_switchboard_to_feems(switchboard: proto.Switchboard) -> Switchboard:
    components = []
    switchboard_id = switchboard.switchboard_id
    for subsystem in switchboard.subsystems:
        if subsystem.component_type == proto.Subsystem.ComponentType.FUEL_CELL_SYSTEM:
            components.append(
                convert_proto_fuel_cell_system_to_feems(
                    subsystem=subsystem, switchboard_id=switchboard_id
                )
            )
        elif subsystem.component_type == proto.Subsystem.ComponentType.GENSET:
            components.append(
                convert_proto_genset_to_feems(
                    subsystem=subsystem, switchboard_id=switchboard_id
                )
            )
        elif subsystem.component_type == proto.Subsystem.ComponentType.COGES:
            components.append(
                convert_proto_coges_to_feems(
                    subsystem=subsystem, switchboard_id=switchboard_id
                )
            )
        elif subsystem.component_type == proto.Subsystem.ComponentType.BATTERY_SYSTEM:
            components.append(
                convert_proto_battery_system_to_feems(
                    subsystem=subsystem, switchboard_id=switchboard_id
                )
            )
        elif subsystem.component_type == proto.Subsystem.ComponentType.BATTERY:
            components.append(
                convert_proto_battery_to_feems(
                    proto_component=subsystem.battery, switchboard_id=switchboard_id
                )
            )
        elif (
            subsystem.component_type
            == proto.Subsystem.ComponentType.SUPERCAPACITOR_SYSTEM
        ):
            components.append(
                convert_proto_supercapacitor_system_to_feems(
                    subsystem=subsystem, switchboard_id=switchboard_id
                )
            )
        elif subsystem.component_type == proto.Subsystem.ComponentType.SUPERCAPACITOR:
            components.append(
                convert_proto_supercapacitor_to_feems(
                    proto_component=subsystem.supercapacitor,
                    switchboard_id=switchboard_id,
                )
            )
        else:
            components.append(
                convert_generic_electric_subsystem_to_feems(
                    subsystem=subsystem, switchboard_id=switchboard_id
                )
            )

    return Switchboard(
        name=f"SWBD {switchboard.switchboard_id}",
        idx=switchboard.switchboard_id,
        components=components,
    )


def convert_proto_shaftline_to_feems(
    shaftline: proto.ShaftLine,
    pti_ptos: Optional[List[PTIPTO]] = None,
) -> ShaftLine:
    shaft_line_id = shaftline.shaft_line_id
    components = []
    for sub_system in shaftline.subsystems:
        if sub_system.component_type == proto.Subsystem.ComponentType.MAIN_ENGINE:
            if sub_system.engine.name == "":
                sub_system.engine.name = sub_system.name
            components.append(
                MainEngineForMechanicalPropulsion(
                    name=sub_system.name,
                    engine=convert_proto_engine_to_feems(
                        proto_engine=sub_system.engine,
                        type_engine=TypeComponent.MAIN_ENGINE,
                    ),
                    shaft_line_id=shaft_line_id,
                )
            )
        elif (
            sub_system.component_type
            == proto.Subsystem.ComponentType.MAIN_ENGINE_WITH_GEARBOX
        ):
            components.append(
                MainEngineWithGearBoxForMechanicalPropulsion(
                    name=sub_system.name,
                    main_engine=convert_proto_engine_to_feems(
                        proto_engine=sub_system.engine,
                        type_engine=TypeComponent.MAIN_ENGINE_WITH_GEARBOX,
                    ),
                    gearbox=BasicComponent(
                        type_=TypeComponent.GEARBOX,
                        name=sub_system.gearbox.name,
                        power_type=TypePower.POWER_TRANSMISSION,
                        rated_power=sub_system.gear.rated_power_kw,
                        rated_speed=sub_system.gear.rated_speed_rpm,
                        eff_curve=convert_proto_efficiency_bsfc_power_to_np_array(
                            efficiency_bsfc_power=sub_system.gear.efficiency
                        ),
                    ),
                    shaft_line_id=shaft_line_id,
                )
            )
        elif sub_system.component_type == proto.Subsystem.ComponentType.PTI_PTO_SYSTEM:
            if pti_ptos is None:
                components.append(
                    convert_proto_pti_pto_subsystem_to_feems(
                        subsystem=sub_system,
                        switchboard_id=1,
                        shaft_line_id=shaft_line_id,
                    )
                )
            else:
                try:
                    pti_pto = next(
                        filter(
                            lambda pti_pto: pti_pto.name == sub_system.name, pti_ptos
                        )
                    )
                except StopIteration as e:
                    print(
                        f"PTI/PTO {sub_system.name} not found in pti_ptos given as argument."
                    )
                    print("Creating a new PTI/PTO from the proto definition.")
                    components.append(
                        convert_proto_pti_pto_subsystem_to_feems(
                            subsystem=sub_system,
                            switchboard_id=1,
                            shaft_line_id=shaft_line_id,
                        )
                    )
                else:
                    components.append(pti_pto)
        elif sub_system.component_type == proto.Subsystem.ComponentType.PROPELLER_LOAD:
            components.append(
                MechanicalPropulsionComponent(
                    type_=TypeComponent.PROPELLER_LOAD,
                    power_type=TypePower.POWER_CONSUMER,
                    name=sub_system.name,
                    shaft_line_id=shaft_line_id,
                    rated_power=sub_system.rated_power_kw,
                    rated_speed=sub_system.rated_speed_rpm,
                    eff_curve=convert_proto_efficiency_bsfc_power_to_np_array(
                        efficiency_bsfc_power=sub_system.propeller.efficiency
                    ),
                )
            )
        else:
            raise ValueError(
                f"The component type {TypeComponent(sub_system.component_type)} "
                f"is not supported."
            )
    return ShaftLine(
        name=f"Shaftline {shaftline.shaft_line_id}",
        shaft_line_id=shaftline.shaft_line_id,
        component_list=components,
    )


# %% ../00_ConvertToFeems.ipynb 8
def convert_feems_switchboards_to_feems_electric_power_system(
    switchboards: List[Switchboard],
) -> ElectricPowerSystem:
    def get_all_components(switchboard: Switchboard) -> List[Any]:
        return reduce(
            lambda components1, components2: [*components1, *components2],
            switchboard.component_by_power_type,
            [],
        )

    components = reduce(
        lambda acc, switchboard: [
            *acc,
            *get_all_components(switchboard),
        ],
        switchboards,
        [],
    )
    bus_tie_connection = [
        (index + 1, index + 2) for index in range(len(switchboards) - 1)
    ]
    return ElectricPowerSystem(
        name="electric power system",
        power_plant_components=components,
        bus_tie_connections=bus_tie_connection,
    )


def convert_feems_shaftlines_to_feems_mechanical_propulsion_system(
    shaftlines: List[ShaftLine],
) -> MechanicalPropulsionSystem:

    components = reduce(
        lambda acc, shaftline: [*acc, *shaftline.components], shaftlines, []
    )
    return MechanicalPropulsionSystem(
        name="mechanical propulsion system",
        components_list=components,
    )


def convert_proto_electric_system_to_feems(
    system: proto.ElectricSystem,
) -> ElectricPowerSystem:
    switchboards = [
        convert_proto_switchboard_to_feems(proto_switchboard)
        for proto_switchboard in system.switchboards
    ]
    return convert_feems_switchboards_to_feems_electric_power_system(switchboards)


def convert_proto_mechanical_system_to_feems(
    system: proto.MechanicalSystem, pti_ptos: List[PTIPTO] = None
) -> MechanicalPropulsionSystem:
    shaft_lines = []
    for proto_shaftline in system.shaft_lines:
        if pti_ptos is not None:
            pti_ptos_for_shaft_lines = list(
                filter(
                    lambda pti_pto: pti_pto.shaft_line_id
                    == proto_shaftline.shaft_line_id,
                    pti_ptos,
                )
            )
        else:
            pti_ptos_for_shaft_lines = None
        shaft_lines.append(
            convert_proto_shaftline_to_feems(
                shaftline=proto_shaftline, pti_ptos=pti_ptos_for_shaft_lines
            )
        )
    return convert_feems_shaftlines_to_feems_mechanical_propulsion_system(
        shaftlines=shaft_lines
    )


def convert_proto_propulsion_system_to_feems(
    system: proto.MachinerySystem,
) -> Union[
    MechanicalPropulsionSystemWithElectricPowerSystem,
    ElectricPowerSystem,
    HybridPropulsionSystem,
    None,
]:
    if system.propulsion_type == proto.MachinerySystem.PropulsionType.MECHANICAL:
        return MechanicalPropulsionSystemWithElectricPowerSystem(
            name=system.name,
            electric_system=convert_proto_electric_system_to_feems(
                system.electric_system
            ),
            mechanical_system=convert_proto_mechanical_system_to_feems(
                system.mechanical_system
            ),
        )
    if system.propulsion_type == proto.MachinerySystem.PropulsionType.ELECTRIC:
        return convert_proto_electric_system_to_feems(system.electric_system)
    if system.propulsion_type == proto.MachinerySystem.PropulsionType.HYBRID:
        electric_system = convert_proto_electric_system_to_feems(system.electric_system)
        pti_ptos = electric_system.pti_pto if len(electric_system.pti_pto) > 0 else None
        return HybridPropulsionSystem(
            name=system.name,
            electric_system=electric_system,
            mechanical_system=convert_proto_mechanical_system_to_feems(
                system=system.mechanical_system, pti_ptos=pti_ptos
            ),
        )
    return None



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feems_result.proto
# Protobuf Python Version: 4.25.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import system_structure_pb2 as system__structure__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x12\x66\x65\x65ms_result.proto\x12\x0b\x66\x65\x65msResult\x1a\x16system_structure.proto"\xdb\x02\n\nFuelScalar\x12\x35\n\tfuel_type\x18\x01 \x01(\x0e\x32".machinerySystemStructure.FuelType\x12\x39\n\x0b\x66uel_origin\x18\x02 \x01(\x0e\x32$.machinerySystemStructure.FuelOrigin\x12\x44\n\x11\x66uel_specified_by\x18\x03 \x01(\x0e\x32).machinerySystemStructure.FuelSpecifiedBy\x12\x1d\n\x15mass_or_mass_fraction\x18\x04 \x01(\x01\x12\x14\n\x0clhv_mj_per_g\x18\x05 \x01(\x01\x12\x36\n.ghg_emission_factor_well_to_tank_gco2eq_per_mj\x18\x06 \x01(\x01\x12(\n ghg_emission_factor_tank_to_wake\x18\x07 \x01(\x01"\xda\x02\n\tFuelArray\x12\x35\n\tfuel_type\x18\x01 \x01(\x0e\x32".machinerySystemStructure.FuelType\x12\x39\n\x0b\x66uel_origin\x18\x02 \x01(\x0e\x32$.machinerySystemStructure.FuelOrigin\x12\x44\n\x11\x66uel_specified_by\x18\x03 \x01(\x0e\x32).machinerySystemStructure.FuelSpecifiedBy\x12\x1d\n\x15mass_or_mass_fraction\x18\x04 \x03(\x01\x12\x14\n\x0clhv_mj_per_g\x18\x05 \x01(\x01\x12\x36\n.ghg_emission_factor_well_to_tank_gco2eq_per_mj\x18\x06 \x01(\x01\x12(\n ghg_emission_factor_tank_to_wake\x18\x07 \x01(\x01"?\n\x15\x46uelConsumptionScalar\x12&\n\x05\x66uels\x18\x01 \x03(\x0b\x32\x17.feemsResult.FuelScalar"A\n\x18\x46uelConsumptionRateArray\x12%\n\x05\x66uels\x18\x01 \x03(\x0b\x32\x16.feemsResult.FuelArray"\x8f\x01\n\x1cTimeSeriesResultForComponent\x12\x0c\n\x04time\x18\x01 \x03(\x01\x12\x17\n\x0fpower_output_kw\x18\x02 \x03(\x01\x12H\n\x19\x66uel_consumption_kg_per_s\x18\x03 \x01(\x0b\x32%.feemsResult.FuelConsumptionRateArray"\xee\x03\n\x12ResultPerComponent\x12\x16\n\x0e\x63omponent_name\x18\x01 \x01(\t\x12\x45\n\x19multi_fuel_consumption_kg\x18\x02 \x01(\x0b\x32".feemsResult.FuelConsumptionScalar\x12&\n\x1e\x65lectric_energy_consumption_mj\x18\x03 \x01(\x01\x12(\n mechanical_energy_consumption_mj\x18\x04 \x01(\x01\x12\x18\n\x10\x65nergy_stored_mj\x18\x05 \x01(\x01\x12\x17\n\x0frunning_hours_h\x18\x06 \x01(\x01\x12\x18\n\x10\x63o2_emissions_kg\x18\x07 \x01(\x01\x12\x18\n\x10nox_emissions_kg\x18\x08 \x01(\x01\x12\x16\n\x0e\x63omponent_type\x18\t \x01(\t\x12\x16\n\x0erated_capacity\x18\n \x01(\x01\x12\x1b\n\x13rated_capacity_unit\x18\x0b \x01(\t\x12\x16\n\x0eswitchboard_id\x18\x0c \x01(\r\x12\x14\n\x0cshaftline_id\x18\r \x01(\r\x12\x45\n\x12result_time_series\x18\x0e \x01(\x0b\x32).feemsResult.TimeSeriesResultForComponent"\xb5\x05\n\x0b\x46\x65\x65msResult\x12\x12\n\nduration_s\x18\x01 \x01(\x01\x12K\n\x1fmulti_fuel_consumption_total_kg\x18\x02 \x01(\x0b\x32".feemsResult.FuelConsumptionScalar\x12,\n$energy_consumption_electric_total_mj\x18\x03 \x01(\x01\x12.\n&energy_consumption_mechanical_total_mj\x18\x04 \x01(\x01\x12\x1e\n\x16\x65nergy_stored_total_mj\x18\x05 \x01(\x01\x12%\n\x1drunning_hours_main_engines_hr\x18\x06 \x01(\x01\x12%\n\x1drunning_hours_genset_total_hr\x18\x07 \x01(\x01\x12(\n running_hours_fuel_cell_total_hr\x18\x08 \x01(\x01\x12&\n\x1erunning_hours_pti_pto_total_hr\x18\t \x01(\x01\x12\x1d\n\x15\x63o2_emission_total_kg\x18\n \x01(\x01\x12\x1d\n\x15nox_emission_total_kg\x18\x0b \x01(\x01\x12\x38\n\x0f\x64\x65tailed_result\x18\x0c \x03(\x0b\x32\x1f.feemsResult.ResultPerComponent\x12(\n energy_input_mechanical_total_mj\x18\r \x01(\x01\x12&\n\x1e\x65nergy_input_electric_total_mj\x18\x0e \x01(\x01\x12.\n&energy_consumption_propulsion_total_mj\x18\x0f \x01(\x01\x12-\n%energy_consumption_auxiliary_total_mj\x18\x10 \x01(\x01"\x87\x01\n\x1d\x46\x65\x65msResultForMachinerySystem\x12\x31\n\x0f\x65lectric_system\x18\x01 \x01(\x0b\x32\x18.feemsResult.FeemsResult\x12\x33\n\x11mechanical_system\x18\x02 \x01(\x0b\x32\x18.feemsResult.FeemsResultb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "feems_result_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_FUELSCALAR"]._serialized_start = 60
    _globals["_FUELSCALAR"]._serialized_end = 407
    _globals["_FUELARRAY"]._serialized_start = 410
    _globals["_FUELARRAY"]._serialized_end = 756
    _globals["_FUELCONSUMPTIONSCALAR"]._serialized_start = 758
    _globals["_FUELCONSUMPTIONSCALAR"]._serialized_end = 821
    _globals["_FUELCONSUMPTIONRATEARRAY"]._serialized_start = 823
    _globals["_FUELCONSUMPTIONRATEARRAY"]._serialized_end = 888
    _globals["_TIMESERIESRESULTFORCOMPONENT"]._serialized_start = 891
    _globals["_TIMESERIESRESULTFORCOMPONENT"]._serialized_end = 1034
    _globals["_RESULTPERCOMPONENT"]._serialized_start = 1037
    _globals["_RESULTPERCOMPONENT"]._serialized_end = 1531
    _globals["_FEEMSRESULT"]._serialized_start = 1534
    _globals["_FEEMSRESULT"]._serialized_end = 2227
    _globals["_FEEMSRESULTFORMACHINERYSYSTEM"]._serialized_start = 2230
    _globals["_FEEMSRESULTFORMACHINERYSYSTEM"]._serialized_end = 2365
# @@protoc_insertion_point(module_scope)



import os

from MachSysS.utility import retrieve_machinery_system_from_file
from MachSysS.convert_to_feems import convert_proto_propulsion_system_to_feems

if __name__ == "__main__":
    path = os.path.join("electric_propulsion_system.mss")
    system_proto = retrieve_machinery_system_from_file(path)
    system_feems = convert_proto_propulsion_system_to_feems(system_proto)
    print(system_feems.switchboards)



"""
This module contains utility functions to compare proto objects.

Functions:
    compare_proto_components: compare two proto components and return a dict of differences.
    compare_proto_subsystems: compare two proto subsystems and return a dict of differences.
    convert_camel_case_to_snake_case: convert a camel case string to snake case.
    convert_camel_case_list_to_snake_case_list: convert a list of camel case strings to snake case.

History:
    created: 22/02/2021

Creator: Kevin K. Yum
CopyRight: SINTEF Ocean
"""

import re
from dataclasses import dataclass
from functools import reduce
from typing import List, Union, Dict
import MachSysS.system_structure_pb2 as proto
from deepdiff.diff import DeepDiff
from google.protobuf.json_format import MessageToDict


ProtoComponent = Union[
    proto.MechanicalComponent,
    proto.ElectricComponent,
    proto.FuelCell,
]


def convert_camel_case_to_snake_case(camel_case_string: str) -> str:
    """
    Convert a camel case string to snake case.

    :param camel_case_string: string in camel case
    :return: string in snake case
    """
    return re.sub(r"(?<!^)(?=[A-Z])", "_", camel_case_string).lower()


def convert_camel_case_list_to_snake_case_list(camel_case_list: List[str]) -> List[str]:
    """
    Convert a list of camel case strings to snake case.

    :param camel_case_list: list of strings in camel case
    :return: list of strings in snake case
    """
    return [
        convert_camel_case_to_snake_case(camel_case_string)
        for camel_case_string in camel_case_list
    ]


@dataclass
class DiffSubsystems:
    """
    Class to store the differences between two subsystems.
    """

    diff_attributes: DeepDiff
    components_removed: List[str]
    components_added: List[str]
    components_modified: Dict[str, DeepDiff]

    def to_dict(self) -> dict:
        """
        Convert the class to a dict.

        :return: dict representation of the class
        """
        result = self.__dict__
        result["diff_attributes"] = self.diff_attributes.to_dict()
        result["components_modified"] = {
            key: value.to_dict() for key, value in self.components_modified.items()
        }
        return result


@dataclass
class DiffSwitchboardShaftLines:
    """
    Class to store the differences between two switchboards.
    """

    subsystems_removed: List[str]
    subsystems_added: List[str]
    subsystems_modified: Dict[str, DiffSubsystems]

    def to_dict(self) -> dict:
        """
        Convert the class to a dict.

        :return: dict representation of the class
        """
        result = self.__dict__
        result["subsystems_modified"] = {
            name: diff.to_dict() for name, diff in self.subsystems_modified.items()
        }
        return result


@dataclass
class DiffElectricPowerSystem:
    switchboards_removed: List[int]
    switchboards_added: List[int]
    switchboards_modified: Dict[int, DiffSwitchboardShaftLines]


@dataclass
class DiffMechanicalSystem:
    shaft_lines_removed: List[int]
    shaft_lines_added: List[int]
    shaft_lines_modified: Dict[int, DiffSwitchboardShaftLines]


@dataclass
class DiffMachinerySystem:
    diff_attributes: DeepDiff
    diff_electric_system: DiffElectricPowerSystem
    diff_mechanical_system: DiffMechanicalSystem


def compare_proto_components(comp1: ProtoComponent, comp2: ProtoComponent) -> DeepDiff:
    """
    Compare two proto components and return a dict of differences.

    :param comp1: first component to compare. It should be a basic proto component.
    :param comp2: second component to compare. It should be a basic proto component.
    :return: DeepDiff object
    """
    comp1_dict = MessageToDict(comp1)
    comp2_dict = MessageToDict(comp2)
    return DeepDiff(
        t1=comp1_dict,
        t2=comp2_dict,
        ignore_order=True,
        report_repetition=True,
        ignore_string_case=True,
    )


def compare_proto_subsystems(
    subsystem1: proto.Subsystem, subsystem2: proto.Subsystem
) -> DiffSubsystems:
    """
    Compare two proto subsystems and return a dict of differences.

    :param subsystem1: first subsystem to compare. It should be a basic proto subsystem.
    :param subsystem2: second subsystem to compare. It should be a basic proto subsystem.
    :return: dict of differences
    """
    non_component_attribute_keys = [
        "powerType",
        "componentType",
        "name",
        "ratedPowerKw",
        "ratedSpeedRpm",
    ]
    subsystem1_dict = MessageToDict(subsystem1)
    subsystem2_dict = MessageToDict(subsystem2)
    subsystem1_dict_non_components = reduce(
        lambda acc, key: (
            {**acc, **{key: subsystem1_dict.pop(key)}}
            if key in subsystem1_dict
            else acc
        ),
        non_component_attribute_keys,
        {},
    )
    subsystem2_dict_non_components = reduce(
        lambda acc, key: (
            {**acc, **{key: subsystem2_dict.pop(key)}}
            if key in subsystem2_dict
            else acc
        ),
        non_component_attribute_keys,
        {},
    )
    diff_non_components = DeepDiff(
        subsystem1_dict_non_components,
        subsystem2_dict_non_components,
        ignore_order=True,
        report_repetition=True,
    )
    components_list_subsystem1 = convert_camel_case_list_to_snake_case_list(
        list(subsystem1_dict.keys())
    )
    components_list_subsystem2 = convert_camel_case_list_to_snake_case_list(
        list(subsystem2_dict.keys())
    )
    components_removed = list(
        set(components_list_subsystem1) - set(components_list_subsystem2)
    )
    components_added = list(
        set(components_list_subsystem2) - set(components_list_subsystem1)
    )
    components_common = list(
        set(components_list_subsystem1) & set(components_list_subsystem2)
    )
    diff_components = {}
    for component_name in components_common:
        diff_components[component_name] = compare_proto_components(
            getattr(subsystem1, component_name), getattr(subsystem2, component_name)
        )
    return DiffSubsystems(
        diff_attributes=diff_non_components,
        components_removed=components_removed,
        components_added=components_added,
        components_modified=diff_components,
    )


def compare_proto_switchboards_shaft_lines(
    switchboard1: proto.Switchboard, switchboard2: proto.Switchboard
) -> DiffSwitchboardShaftLines:
    """
    Compare two proto switchboards and return a dict of differences.

    :param switchboard1: first switchboard to compare. It should be a basic proto switchboard.
    :param switchboard2: second switchboard to compare. It should be a basic proto switchboard.
    :return: dict of differences
    """
    diff = DiffSwitchboardShaftLines(
        subsystems_removed=[],
        subsystems_added=[subsystem.name for subsystem in switchboard2.subsystems],
        subsystems_modified={},
    )
    for subsystem_ref in switchboard1.subsystems:
        try:
            subsystem_to_compare = next(
                filter(
                    lambda subsystem: subsystem_ref.name == subsystem.name,
                    switchboard2.subsystems,
                )
            )
        except StopIteration:
            diff.subsystems_removed.append(subsystem_ref.name)
        else:
            diff.subsystems_added.remove(subsystem_ref.name)
            diff_subsystem = compare_proto_subsystems(
                subsystem_ref, subsystem_to_compare
            )
            diff.subsystems_modified[subsystem_ref.name] = diff_subsystem
    return diff


def compare_proto_electric_systems(
    eps1: proto.ElectricSystem, eps2: proto.ElectricSystem
) -> DiffElectricPowerSystem:
    """
    Compare two proto electric power systems and return a dict of differences.

    :param eps1: first electric power system to compare. It should be a basic proto electric power system.
    :param eps2: second electric power system to compare. It should be a basic proto electric power system.
    :return: DiffElectricPowerSystem object
    """
    diff = DiffElectricPowerSystem(
        switchboards_removed=[],
        switchboards_added=[
            switchboard.switchboard_id for switchboard in eps2.switchboards
        ],
        switchboards_modified={},
    )
    for switchboard_ref in eps1.switchboards:
        try:
            switchboard_to_compare = next(
                filter(
                    lambda switchboard: switchboard_ref.switchboard_id
                    == switchboard.switchboard_id,
                    eps2.switchboards,
                )
            )
        except StopIteration:
            diff.switchboards_removed.append(switchboard_ref.switchboard_id)
        else:
            diff.switchboards_added.remove(switchboard_ref.switchboard_id)
            diff_switchboard = compare_proto_switchboards_shaft_lines(
                switchboard_ref, switchboard_to_compare
            )
            diff.switchboards_modified[switchboard_ref.switchboard_id] = (
                diff_switchboard
            )
    return diff


def compare_proto_mechanical_system(
    shaft_lines1: proto.ShaftLine, shaft_lines2: proto.ShaftLine
) -> DiffMechanicalSystem:
    """
    Compare two proto shaft lines and return a dict of differences.

    :param shaft_lines1: first shaft lines to compare. It should be a basic proto shaft lines.
    :param shaft_lines2: second shaft lines to compare. It should be a basic proto shaft lines.
    :return: DiffMechanicalSystem object
    """
    diff = DiffMechanicalSystem(
        shaft_lines_removed=[],
        shaft_lines_added=[
            shaft_line.shaft_line_id for shaft_line in shaft_lines2.shaft_lines
        ],
        shaft_lines_modified={},
    )
    for shaft_line_ref in shaft_lines1.shaft_lines:
        try:
            shaft_line_to_compare = next(
                filter(
                    lambda shaft_line: shaft_line_ref.shaft_line_id
                    == shaft_line.shaft_line_id,
                    shaft_lines2.shaft_lines,
                )
            )
        except StopIteration:
            diff.shaft_lines_removed.append(shaft_line_ref.shaft_line_id)
        else:
            diff.shaft_lines_added.remove(shaft_line_ref.shaft_line_id)
            diff_shaft_line = compare_proto_switchboards_shaft_lines(
                shaft_line_ref, shaft_line_to_compare
            )
            diff.shaft_lines_modified[shaft_line_ref.shaft_line_id] = diff_shaft_line
    return diff


def compare_proto_machinery_system(
    machinery1: proto.MachinerySystem, machinery2: proto.MachinerySystem
) -> DiffMachinerySystem:
    """
    Compare two proto machinery systems and return a dict of differences.

    :param machinery1: first machinery system to compare. It should be a basic proto machinery system.
    :param machinery2: second machinery system to compare. It should be a basic proto machinery system.
    :return: DiffMachinerySystem object
    """
    machinery_system1_dict = MessageToDict(machinery1)
    machinery_system2_dict = MessageToDict(machinery2)
    machinery_system1_dict.pop("electricSystem", None)
    machinery_system2_dict.pop("electricSystem", None)
    machinery_system1_dict.pop("mechanicalSystem", None)
    machinery_system2_dict.pop("mechanicalSystem", None)
    return DiffMachinerySystem(
        diff_attributes=DeepDiff(machinery_system1_dict, machinery_system2_dict),
        diff_electric_system=compare_proto_electric_systems(
            machinery1.electric_system, machinery2.electric_system
        ),
        diff_mechanical_system=compare_proto_mechanical_system(
            machinery1.mechanical_system, machinery2.mechanical_system
        ),
    )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: gymir_result.proto
# Protobuf Python Version: 4.25.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x12gymir_result.proto\x12\x0bgymirResult"g\n\x0bGymirResult\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x19\n\x11\x61uxiliary_load_kw\x18\x02 \x01(\x01\x12/\n\x06result\x18\x03 \x03(\x0b\x32\x1f.gymirResult.SimulationInstance"\xd3\x04\n\x12SimulationInstance\x12\x0f\n\x07\x65poch_s\x18\x01 \x01(\x01\x12\x11\n\ttask_type\x18\x02 \x01(\t\x12\x11\n\ttask_name\x18\x03 \x01(\t\x12\x14\n\x0clatitude_deg\x18\x04 \x01(\x01\x12\x15\n\rlongitude_deg\x18\x05 \x01(\x01\x12\x13\n\x0bheading_deg\x18\x06 \x01(\x01\x12!\n\x19wave_height_significant_m\x18\x07 \x01(\x01\x12\x1a\n\x12wave_peak_period_s\x18\x08 \x01(\x01\x12\x1e\n\x16wave_dir_rel_north_deg\x18\t \x01(\x01\x12\x1f\n\x17wave_dir_rel_vessel_deg\x18\n \x01(\x01\x12\x16\n\x0ewind_speed_mps\x18\x0b \x01(\x01\x12\x1e\n\x16wind_dir_rel_north_deg\x18\x0c \x01(\x01\x12\x1f\n\x17wind_dir_rel_vessel_deg\x18\r \x01(\x01\x12\x16\n\x0eweather_source\x18\x0e \x01(\t\x12\x1c\n\x14speed_over_ground_kn\x18\x0f \x01(\x01\x12\x1e\n\x16speed_through_water_kn\x18\x10 \x01(\x01\x12\x19\n\x11\x63urrent_speed_mps\x18\x11 \x01(\x01\x12!\n\x19\x63urrent_dir_rel_north_deg\x18\x12 \x01(\x01\x12\x10\n\x08power_kw\x18\x13 \x01(\x01\x12\x13\n\x0btorque_k_nm\x18\x14 \x01(\x01\x12\x12\n\nthrust_k_n\x18\x15 \x01(\x01\x12\x1c\n\x14total_resistance_k_n\x18\x16 \x01(\x01"c\n\x17PropulsionPowerInstance\x12\x0f\n\x07\x65poch_s\x18\x01 \x01(\x01\x12\x1b\n\x13propulsion_power_kw\x18\x02 \x01(\x01\x12\x1a\n\x12\x61uxiliary_power_kw\x18\x03 \x01(\x01"K\n\x15OperationProfilePoint\x12\x0f\n\x07\x65poch_s\x18\x01 \x01(\x01\x12\x10\n\x08speed_kn\x18\x02 \x01(\x01\x12\x0f\n\x07\x64raft_m\x18\x03 \x01(\x01"\xb8\x01\n\x10TimeSeriesResult\x12I\n\x1bpropulsion_power_timeseries\x18\x01 \x03(\x0b\x32$.gymirResult.PropulsionPowerInstance\x12\x1a\n\x12\x61uxiliary_power_kw\x18\x02 \x01(\x01\x12=\n\x11operation_profile\x18\x03 \x03(\x0b\x32".gymirResult.OperationProfilePointb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "gymir_result_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_GYMIRRESULT"]._serialized_start = 35
    _globals["_GYMIRRESULT"]._serialized_end = 138
    _globals["_SIMULATIONINSTANCE"]._serialized_start = 141
    _globals["_SIMULATIONINSTANCE"]._serialized_end = 736
    _globals["_PROPULSIONPOWERINSTANCE"]._serialized_start = 738
    _globals["_PROPULSIONPOWERINSTANCE"]._serialized_end = 837
    _globals["_OPERATIONPROFILEPOINT"]._serialized_start = 839
    _globals["_OPERATIONPROFILEPOINT"]._serialized_end = 914
    _globals["_TIMESERIESRESULT"]._serialized_start = 917
    _globals["_TIMESERIESRESULT"]._serialized_end = 1101
# @@protoc_insertion_point(module_scope)



# Autogenerated by nbdev

d = {
    "settings": {
        "branch": "master",
        "doc_baseurl": "/MachSysS/",
        "doc_host": "https://keviny.github.io",
        "git_url": "https://SintefOceanEnergySystem@dev.azure.com/SintefOceanEnergySystem/MachinerySystemStucture/_git/MachinerySystemStucture",
        "lib_path": "MachSysS",
    },
    "syms": {
        "MachSysS.convert_feems_result_to_proto": {
            "MachSysS.convert_feems_result_to_proto.FEEMSResultConverter": (
                "convertfeemsresulttoproto.html#feemsresultconverter",
                "MachSysS/convert_feems_result_to_proto.py",
            ),
            "MachSysS.convert_feems_result_to_proto.FEEMSResultConverter.__init__": (
                "convertfeemsresulttoproto.html#feemsresultconverter.__init__",
                "MachSysS/convert_feems_result_to_proto.py",
            ),
            "MachSysS.convert_feems_result_to_proto.FEEMSResultConverter._get_feems_result_proto_for_subsystem": (
                "convertfeemsresulttoproto.html#feemsresultconverter._get_feems_result_proto_for_subsystem",
                "MachSysS/convert_feems_result_to_proto.py",
            ),
            "MachSysS.convert_feems_result_to_proto.FEEMSResultConverter._retrieve_time_series_data_from_components": (
                "convertfeemsresulttoproto.html#feemsresultconverter._retrieve_time_series_data_from_components",
                "MachSysS/convert_feems_result_to_proto.py",
            ),
            "MachSysS.convert_feems_result_to_proto.FEEMSResultConverter._time_interval_to_time_array": (
                "convertfeemsresulttoproto.html#feemsresultconverter._time_interval_to_time_array",
                "MachSysS/convert_feems_result_to_proto.py",
            ),
            "MachSysS.convert_feems_result_to_proto.FEEMSResultConverter.electric_system": (
                "convertfeemsresulttoproto.html#feemsresultconverter.electric_system",
                "MachSysS/convert_feems_result_to_proto.py",
            ),
            "MachSysS.convert_feems_result_to_proto.FEEMSResultConverter.get_feems_result_proto": (
                "convertfeemsresulttoproto.html#feemsresultconverter.get_feems_result_proto",
                "MachSysS/convert_feems_result_to_proto.py",
            ),
            "MachSysS.convert_feems_result_to_proto.FEEMSResultConverter.get_timeseries_for_power_sources_and_energy_storage": (
                "convertfeemsresulttoproto.html#feemsresultconverter.get_timeseries_for_power_sources_and_energy_storage",
                "MachSysS/convert_feems_result_to_proto.py",
            ),
            "MachSysS.convert_feems_result_to_proto.FEEMSResultConverter.isSystemElectric": (
                "convertfeemsresulttoproto.html#feemsresultconverter.issystemelectric",
                "MachSysS/convert_feems_result_to_proto.py",
            ),
            "MachSysS.convert_feems_result_to_proto.FEEMSResultConverter.mechanical_system": (
                "convertfeemsresulttoproto.html#feemsresultconverter.mechanical_system",
                "MachSysS/convert_feems_result_to_proto.py",
            ),
            "MachSysS.convert_feems_result_to_proto.FEEMSResultConverter.system_feems": (
                "convertfeemsresulttoproto.html#feemsresultconverter.system_feems",
                "MachSysS/convert_feems_result_to_proto.py",
            ),
        },
        "MachSysS.convert_gymir_result_to_proto": {
            "MachSysS.convert_gymir_result_to_proto.GymirResultConverter": (
                "convertgymirresulttoproto.html#gymirresultconverter",
                "MachSysS/convert_gymir_result_to_proto.py",
            ),
            "MachSysS.convert_gymir_result_to_proto.GymirResultConverter.__init__": (
                "convertgymirresulttoproto.html#gymirresultconverter.__init__",
                "MachSysS/convert_gymir_result_to_proto.py",
            ),
            "MachSysS.convert_gymir_result_to_proto.GymirResultConverter.read_csv": (
                "convertgymirresulttoproto.html#gymirresultconverter.read_csv",
                "MachSysS/convert_gymir_result_to_proto.py",
            ),
            "MachSysS.convert_gymir_result_to_proto.GymirResultConverter.to_csv": (
                "convertgymirresulttoproto.html#gymirresultconverter.to_csv",
                "MachSysS/convert_gymir_result_to_proto.py",
            ),
            "MachSysS.convert_gymir_result_to_proto.find_csv_name_from_message_field_name": (
                "convertgymirresulttoproto.html#find_csv_name_from_message_field_name",
                "MachSysS/convert_gymir_result_to_proto.py",
            ),
            "MachSysS.convert_gymir_result_to_proto.get_message_type_for_simulation_instance": (
                "convertgymirresulttoproto.html#get_message_type_for_simulation_instance",
                "MachSysS/convert_gymir_result_to_proto.py",
            ),
        },
        "MachSysS.convert_proto_timeseries": {
            "MachSysS.convert_proto_timeseries.convert_proto_timeseries_to_pd_dataframe": (
                "converttimeseriesresultprototopandas.html#convert_proto_timeseries_to_pd_dataframe",
                "MachSysS/convert_proto_timeseries.py",
            ),
            "MachSysS.convert_proto_timeseries.convert_proto_timeseries_to_pd_series": (
                "converttimeseriesresultprototopandas.html#convert_proto_timeseries_to_pd_series",
                "MachSysS/convert_proto_timeseries.py",
            ),
        },
        "MachSysS.convert_to_feems": {
            "MachSysS.convert_to_feems.collect_electric_components_from_sub_system": (
                "converttofeems.html#collect_electric_components_from_sub_system",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_emission_curve_to_feems": (
                "converttofeems.html#convert_emission_curve_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_feems_shaftlines_to_feems_mechanical_propulsion_system": (
                "converttofeems.html#convert_feems_shaftlines_to_feems_mechanical_propulsion_system",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_feems_switchboards_to_feems_electric_power_system": (
                "converttofeems.html#convert_feems_switchboards_to_feems_electric_power_system",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_generic_electric_subsystem_to_feems": (
                "converttofeems.html#convert_generic_electric_subsystem_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_nox_calculation_method": (
                "converttofeems.html#convert_nox_calculation_method",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_battery_system_to_feems": (
                "converttofeems.html#convert_proto_battery_system_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_battery_to_feems": (
                "converttofeems.html#convert_proto_battery_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_cogas_to_feems": (
                "converttofeems.html#convert_proto_cogas_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_coges_to_feems": (
                "converttofeems.html#convert_proto_coges_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_curve1d_to_np_array": (
                "converttofeems.html#convert_proto_curve1d_to_np_array",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_efficiency_bsfc_power_to_np_array": (
                "converttofeems.html#convert_proto_efficiency_bsfc_power_to_np_array",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_electric_component_to_feems": (
                "converttofeems.html#convert_proto_electric_component_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_electric_machine_to_feems": (
                "converttofeems.html#convert_proto_electric_machine_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_electric_system_to_feems": (
                "converttofeems.html#convert_proto_electric_system_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_engine_to_feems": (
                "converttofeems.html#convert_proto_engine_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_fuel_cell_system_to_feems": (
                "converttofeems.html#convert_proto_fuel_cell_system_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_generic_electric_component_to_feems": (
                "converttofeems.html#convert_proto_generic_electric_component_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_genset_to_feems": (
                "converttofeems.html#convert_proto_genset_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_mechanical_system_to_feems": (
                "converttofeems.html#convert_proto_mechanical_system_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_point_to_list": (
                "converttofeems.html#convert_proto_point_to_list",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_propulsion_system_to_feems": (
                "converttofeems.html#convert_proto_propulsion_system_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_pti_pto_subsystem_to_feems": (
                "converttofeems.html#convert_proto_pti_pto_subsystem_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_serial_subsystem_to_feems": (
                "converttofeems.html#convert_proto_serial_subsystem_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_shaftline_to_feems": (
                "converttofeems.html#convert_proto_shaftline_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_supercapacitor_system_to_feems": (
                "converttofeems.html#convert_proto_supercapacitor_system_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_supercapacitor_to_feems": (
                "converttofeems.html#convert_proto_supercapacitor_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_switchboard_to_feems": (
                "converttofeems.html#convert_proto_switchboard_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.get_component_type": (
                "converttofeems.html#get_component_type",
                "MachSysS/convert_to_feems.py",
            ),
        },
        "MachSysS.convert_to_protobuf": {
            "MachSysS.convert_to_protobuf.convert_battery_component_to_protobuf": (
                "converttoprotobuf.html#convert_battery_component_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_bsfc_curve_to_protobuf": (
                "converttoprotobuf.html#convert_bsfc_curve_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_cogas_component_to_protobuf": (
                "converttoprotobuf.html#convert_cogas_component_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_efficiency_curve_to_protobuf": (
                "converttoprotobuf.html#convert_efficiency_curve_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_electric_component_to_protobuf": (
                "converttoprotobuf.html#convert_electric_component_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_electric_machine_to_protobuf": (
                "converttoprotobuf.html#convert_electric_machine_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_electric_system_to_protobuf": (
                "converttoprotobuf.html#convert_electric_system_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_electric_system_to_protobuf_machinery_system": (
                "converttoprotobuf.html#convert_electric_system_to_protobuf_machinery_system",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_emission_curves_to_protobuf": (
                "converttoprotobuf.html#convert_emission_curves_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_engine_component_to_protobuf": (
                "converttoprotobuf.html#convert_engine_component_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_hybrid_propulsion_system_to_protobuf": (
                "converttoprotobuf.html#convert_hybrid_propulsion_system_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_mechanical_propulsion_system_with_electric_system_to_protobuf": (
                "converttoprotobuf.html#convert_mechanical_propulsion_system_with_electric_system_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_mechanical_system_to_protobuf": (
                "converttoprotobuf.html#convert_mechanical_system_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_nox_calculation_method_to_protobuf": (
                "converttoprotobuf.html#convert_nox_calculation_method_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_np_array_to_protobuf_power_curve": (
                "converttoprotobuf.html#convert_np_array_to_protobuf_power_curve",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_serial_electric_system_to_protobuf": (
                "converttoprotobuf.html#convert_serial_electric_system_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_shaftline_to_protobuf": (
                "converttoprotobuf.html#convert_shaftline_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_supercapacitor_component_to_protobuf": (
                "converttoprotobuf.html#convert_supercapacitor_component_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_switchboard_to_protobuf": (
                "converttoprotobuf.html#convert_switchboard_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
        },
        "MachSysS.feems_result_pb2": {},
        "MachSysS.gymir_result_pb2": {},
        "MachSysS.system_structure_pb2": {},
        "MachSysS.utility": {
            "MachSysS.utility.retrieve_gymir_result_from_file": (
                "utility.html#retrieve_gymir_result_from_file",
                "MachSysS/utility.py",
            ),
            "MachSysS.utility.retrieve_machinery_system_from_file": (
                "utility.html#retrieve_machinery_system_from_file",
                "MachSysS/utility.py",
            ),
        },
    },
}



from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import (
    ClassVar as _ClassVar,
    Iterable as _Iterable,
    Mapping as _Mapping,
    Optional as _Optional,
    Union as _Union,
)

DESCRIPTOR: _descriptor.FileDescriptor

class GymirResult(_message.Message):
    __slots__ = ("name", "auxiliary_load_kw", "result")
    NAME_FIELD_NUMBER: _ClassVar[int]
    AUXILIARY_LOAD_KW_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    name: str
    auxiliary_load_kw: float
    result: _containers.RepeatedCompositeFieldContainer[SimulationInstance]
    def __init__(
        self,
        name: _Optional[str] = ...,
        auxiliary_load_kw: _Optional[float] = ...,
        result: _Optional[_Iterable[_Union[SimulationInstance, _Mapping]]] = ...,
    ) -> None: ...

class SimulationInstance(_message.Message):
    __slots__ = (
        "epoch_s",
        "task_type",
        "task_name",
        "latitude_deg",
        "longitude_deg",
        "heading_deg",
        "wave_height_significant_m",
        "wave_peak_period_s",
        "wave_dir_rel_north_deg",
        "wave_dir_rel_vessel_deg",
        "wind_speed_mps",
        "wind_dir_rel_north_deg",
        "wind_dir_rel_vessel_deg",
        "weather_source",
        "speed_over_ground_kn",
        "speed_through_water_kn",
        "current_speed_mps",
        "current_dir_rel_north_deg",
        "power_kw",
        "torque_k_nm",
        "thrust_k_n",
        "total_resistance_k_n",
    )
    EPOCH_S_FIELD_NUMBER: _ClassVar[int]
    TASK_TYPE_FIELD_NUMBER: _ClassVar[int]
    TASK_NAME_FIELD_NUMBER: _ClassVar[int]
    LATITUDE_DEG_FIELD_NUMBER: _ClassVar[int]
    LONGITUDE_DEG_FIELD_NUMBER: _ClassVar[int]
    HEADING_DEG_FIELD_NUMBER: _ClassVar[int]
    WAVE_HEIGHT_SIGNIFICANT_M_FIELD_NUMBER: _ClassVar[int]
    WAVE_PEAK_PERIOD_S_FIELD_NUMBER: _ClassVar[int]
    WAVE_DIR_REL_NORTH_DEG_FIELD_NUMBER: _ClassVar[int]
    WAVE_DIR_REL_VESSEL_DEG_FIELD_NUMBER: _ClassVar[int]
    WIND_SPEED_MPS_FIELD_NUMBER: _ClassVar[int]
    WIND_DIR_REL_NORTH_DEG_FIELD_NUMBER: _ClassVar[int]
    WIND_DIR_REL_VESSEL_DEG_FIELD_NUMBER: _ClassVar[int]
    WEATHER_SOURCE_FIELD_NUMBER: _ClassVar[int]
    SPEED_OVER_GROUND_KN_FIELD_NUMBER: _ClassVar[int]
    SPEED_THROUGH_WATER_KN_FIELD_NUMBER: _ClassVar[int]
    CURRENT_SPEED_MPS_FIELD_NUMBER: _ClassVar[int]
    CURRENT_DIR_REL_NORTH_DEG_FIELD_NUMBER: _ClassVar[int]
    POWER_KW_FIELD_NUMBER: _ClassVar[int]
    TORQUE_K_NM_FIELD_NUMBER: _ClassVar[int]
    THRUST_K_N_FIELD_NUMBER: _ClassVar[int]
    TOTAL_RESISTANCE_K_N_FIELD_NUMBER: _ClassVar[int]
    epoch_s: float
    task_type: str
    task_name: str
    latitude_deg: float
    longitude_deg: float
    heading_deg: float
    wave_height_significant_m: float
    wave_peak_period_s: float
    wave_dir_rel_north_deg: float
    wave_dir_rel_vessel_deg: float
    wind_speed_mps: float
    wind_dir_rel_north_deg: float
    wind_dir_rel_vessel_deg: float
    weather_source: str
    speed_over_ground_kn: float
    speed_through_water_kn: float
    current_speed_mps: float
    current_dir_rel_north_deg: float
    power_kw: float
    torque_k_nm: float
    thrust_k_n: float
    total_resistance_k_n: float
    def __init__(
        self,
        epoch_s: _Optional[float] = ...,
        task_type: _Optional[str] = ...,
        task_name: _Optional[str] = ...,
        latitude_deg: _Optional[float] = ...,
        longitude_deg: _Optional[float] = ...,
        heading_deg: _Optional[float] = ...,
        wave_height_significant_m: _Optional[float] = ...,
        wave_peak_period_s: _Optional[float] = ...,
        wave_dir_rel_north_deg: _Optional[float] = ...,
        wave_dir_rel_vessel_deg: _Optional[float] = ...,
        wind_speed_mps: _Optional[float] = ...,
        wind_dir_rel_north_deg: _Optional[float] = ...,
        wind_dir_rel_vessel_deg: _Optional[float] = ...,
        weather_source: _Optional[str] = ...,
        speed_over_ground_kn: _Optional[float] = ...,
        speed_through_water_kn: _Optional[float] = ...,
        current_speed_mps: _Optional[float] = ...,
        current_dir_rel_north_deg: _Optional[float] = ...,
        power_kw: _Optional[float] = ...,
        torque_k_nm: _Optional[float] = ...,
        thrust_k_n: _Optional[float] = ...,
        total_resistance_k_n: _Optional[float] = ...,
    ) -> None: ...

class PropulsionPowerInstance(_message.Message):
    __slots__ = ("epoch_s", "propulsion_power_kw", "auxiliary_power_kw")
    EPOCH_S_FIELD_NUMBER: _ClassVar[int]
    PROPULSION_POWER_KW_FIELD_NUMBER: _ClassVar[int]
    AUXILIARY_POWER_KW_FIELD_NUMBER: _ClassVar[int]
    epoch_s: float
    propulsion_power_kw: float
    auxiliary_power_kw: float
    def __init__(
        self,
        epoch_s: _Optional[float] = ...,
        propulsion_power_kw: _Optional[float] = ...,
        auxiliary_power_kw: _Optional[float] = ...,
    ) -> None: ...

class OperationProfilePoint(_message.Message):
    __slots__ = ("epoch_s", "speed_kn", "draft_m")
    EPOCH_S_FIELD_NUMBER: _ClassVar[int]
    SPEED_KN_FIELD_NUMBER: _ClassVar[int]
    DRAFT_M_FIELD_NUMBER: _ClassVar[int]
    epoch_s: float
    speed_kn: float
    draft_m: float
    def __init__(
        self,
        epoch_s: _Optional[float] = ...,
        speed_kn: _Optional[float] = ...,
        draft_m: _Optional[float] = ...,
    ) -> None: ...

class TimeSeriesResult(_message.Message):
    __slots__ = (
        "propulsion_power_timeseries",
        "auxiliary_power_kw",
        "operation_profile",
    )
    PROPULSION_POWER_TIMESERIES_FIELD_NUMBER: _ClassVar[int]
    AUXILIARY_POWER_KW_FIELD_NUMBER: _ClassVar[int]
    OPERATION_PROFILE_FIELD_NUMBER: _ClassVar[int]
    propulsion_power_timeseries: _containers.RepeatedCompositeFieldContainer[
        PropulsionPowerInstance
    ]
    auxiliary_power_kw: float
    operation_profile: _containers.RepeatedCompositeFieldContainer[
        OperationProfilePoint
    ]
    def __init__(
        self,
        propulsion_power_timeseries: _Optional[
            _Iterable[_Union[PropulsionPowerInstance, _Mapping]]
        ] = ...,
        auxiliary_power_kw: _Optional[float] = ...,
        operation_profile: _Optional[
            _Iterable[_Union[OperationProfilePoint, _Mapping]]
        ] = ...,
    ) -> None: ...



# AUTOGENERATED! DO NOT EDIT! File to edit: ../04_ConvertGymirResultToProto.ipynb.

# %% auto 0
__all__ = [
    "logger",
    "ch",
    "formatter",
    "vocabulary",
    "find_csv_name_from_message_field_name",
    "get_message_type_for_simulation_instance",
    "GymirResultConverter",
]

# %% ../04_ConvertGymirResultToProto.ipynb 3
from io import BytesIO
from typing import Union
import pandas as pd
import MachSysS.gymir_result_pb2 as proto
import logging
from google.protobuf.descriptor import FieldDescriptor


# Define logger
logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

ch = logging.StreamHandler()
ch.setLevel(logging.INFO)

formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
ch.setFormatter(formatter)

logger.addHandler(ch)

vocabulary = {
    "task": "task_type",
    "assignment": "task_name",
    "latitude": "latitude_deg",
    "longitude": "longitude_deg",
    "heading": "heading_deg",
    "wave height significant": "wave_height_significant_m",
    "wave peak period (tp)": "wave_peak_period_s",
    "wave direction in degrees relative to geographic north": "wave_dir_rel_north_deg",
    "wave direction in degrees relative to vessel heading": "wave_dir_rel_vessel_deg",
    "wind speed in meter per sec": "wind_speed_mps",
    "wind direction in degrees relative to geographic north": "wind_dir_rel_north_deg",
    "wind direction in degrees relative to vessel heading": "wind_dir_rel_vessel_deg",
    "speed over ground": "speed_over_ground_kn",
    "speed": "speed_over_ground_kn",
    "speed through water": "speed_through_water_kn",
    "sea current speed in meter pr sec": "current_speed_mps",
    "sea current direction in degrees relative to geographic north": "current_dir_rel_north_deg",
    "power": "power_kw",
    "torque": "torque_k_nm",
    "thrust": "thrust_k_n",
    "rtot": "total_resistance_k_n",
    "weather source": "weather_source",
}


def find_csv_name_from_message_field_name(name: str) -> Union[str, None]:
    try:
        return next(filter(lambda key: vocabulary.get(key) == name, vocabulary))
    except StopIteration:
        return None


def get_message_type_for_simulation_instance(field_name: str) -> Union[str, None]:
    fields = proto.SimulationInstance.DESCRIPTOR.fields_by_name
    field = fields.get(field_name, None)
    if field is not None:
        return field.message_type if field.message_type is not None else field.type


class GymirResultConverter:
    gymir_result: Union[proto.GymirResult, None] = None

    def __init__(
        self,
        *,
        file: Union[str, BytesIO] = None,
        gymir_result: Union[proto.GymirResult, None] = None,
    ):
        if file is not None:
            self.gymir_result = proto.GymirResult()
            if isinstance(file, str):
                with open(file, "rb") as file_obj:
                    self.gymir_result.ParseFromString(file_obj.read())
            else:
                self.gymir_result.ParseFromString(file.read())

        if gymir_result is not None:
            self.gymir_result = gymir_result

    def to_csv(self, filename: str):
        if self.gymir_result is not None:
            with open(filename, "wt") as file:
                for each_instance in self.gymir_result.result:
                    epoch_s = each_instance.epoch_s
                    for field in proto.SimulationInstance.DESCRIPTOR.fields_by_name:
                        if field != "epoch_s":
                            csv_name = find_csv_name_from_message_field_name(field)
                            if csv_name is None:
                                logger.warning(f"No csv field name found for {field}")
                            else:
                                value = getattr(each_instance, field)
                                if value is not None:
                                    file.write(f"{epoch_s},{csv_name},{value}\n")

    def read_csv(
        self, file: Union[str, BytesIO], auxiliary_load_kw: float = 0, name: str = None
    ):
        df = pd.read_csv(file, header=None)
        self.gymir_result = proto.GymirResult(
            auxiliary_load_kw=auxiliary_load_kw, name=name
        )
        epoch_s = 0
        current_simulation_instance = proto.SimulationInstance()
        for index, row in df.iterrows():
            if epoch_s != row[0]:
                if epoch_s != 0:
                    self.gymir_result.result.append(current_simulation_instance)
                epoch_s = row[0]
                current_simulation_instance = proto.SimulationInstance(epoch_s=epoch_s)
            field_name = vocabulary.get(row[1])
            if field_name is None:
                logger.warning(f"No field name found for {row[1]}")
            else:
                value = row[2]
                value_type = get_message_type_for_simulation_instance(field_name)
                if value_type == FieldDescriptor.TYPE_DOUBLE:
                    value = float(value)
                elif value_type in [
                    FieldDescriptor.TYPE_INT32,
                    FieldDescriptor.TYPE_INT64,
                    FieldDescriptor.TYPE_UINT32,
                    FieldDescriptor.TYPE_UINT64,
                ]:
                    value = int(value)
                elif value_type == FieldDescriptor.TYPE_BOOL:
                    value = value == "true" or value == "True" or value == "TRUE"
                setattr(current_simulation_instance, field_name, value)
        if len(df) > 0:
            self.gymir_result.result.append(current_simulation_instance)



from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import (
    ClassVar as _ClassVar,
    Iterable as _Iterable,
    Mapping as _Mapping,
    Optional as _Optional,
    Union as _Union,
)

DESCRIPTOR: _descriptor.FileDescriptor

class EmissionType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    NONE: _ClassVar[EmissionType]
    SOX: _ClassVar[EmissionType]
    NOX: _ClassVar[EmissionType]
    CO: _ClassVar[EmissionType]
    PM: _ClassVar[EmissionType]
    HC: _ClassVar[EmissionType]
    CH4: _ClassVar[EmissionType]
    N2O: _ClassVar[EmissionType]

class FuelType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    DIESEL: _ClassVar[FuelType]
    HFO: _ClassVar[FuelType]
    NATURAL_GAS: _ClassVar[FuelType]
    HYDROGEN: _ClassVar[FuelType]
    AMMONIA: _ClassVar[FuelType]
    LPG_PROPANE: _ClassVar[FuelType]
    LPG_BUTANE: _ClassVar[FuelType]
    ETHANOL: _ClassVar[FuelType]
    METHANOL: _ClassVar[FuelType]
    LFO: _ClassVar[FuelType]

class FuelOrigin(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    NONE1: _ClassVar[FuelOrigin]
    FOSSIL: _ClassVar[FuelOrigin]
    BIO: _ClassVar[FuelOrigin]
    RENEWABLE_NON_BIO: _ClassVar[FuelOrigin]

class FuelSpecifiedBy(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    NONE2: _ClassVar[FuelSpecifiedBy]
    FUEL_EU_MARITIME: _ClassVar[FuelSpecifiedBy]
    IMO: _ClassVar[FuelSpecifiedBy]
    USER: _ClassVar[FuelSpecifiedBy]

NONE: EmissionType
SOX: EmissionType
NOX: EmissionType
CO: EmissionType
PM: EmissionType
HC: EmissionType
CH4: EmissionType
N2O: EmissionType
DIESEL: FuelType
HFO: FuelType
NATURAL_GAS: FuelType
HYDROGEN: FuelType
AMMONIA: FuelType
LPG_PROPANE: FuelType
LPG_BUTANE: FuelType
ETHANOL: FuelType
METHANOL: FuelType
LFO: FuelType
NONE1: FuelOrigin
FOSSIL: FuelOrigin
BIO: FuelOrigin
RENEWABLE_NON_BIO: FuelOrigin
NONE2: FuelSpecifiedBy
FUEL_EU_MARITIME: FuelSpecifiedBy
IMO: FuelSpecifiedBy
USER: FuelSpecifiedBy

class Point(_message.Message):
    __slots__ = ("x", "y")
    X_FIELD_NUMBER: _ClassVar[int]
    Y_FIELD_NUMBER: _ClassVar[int]
    x: float
    y: float
    def __init__(
        self, x: _Optional[float] = ..., y: _Optional[float] = ...
    ) -> None: ...

class Curve1D(_message.Message):
    __slots__ = ("points",)
    POINTS_FIELD_NUMBER: _ClassVar[int]
    points: _containers.RepeatedCompositeFieldContainer[Point]
    def __init__(
        self, points: _Optional[_Iterable[_Union[Point, _Mapping]]] = ...
    ) -> None: ...

class BSFCCurve(_message.Message):
    __slots__ = ("x_label", "y_label", "curve")
    X_LABEL_FIELD_NUMBER: _ClassVar[int]
    Y_LABEL_FIELD_NUMBER: _ClassVar[int]
    CURVE_FIELD_NUMBER: _ClassVar[int]
    x_label: str
    y_label: str
    curve: Curve1D
    def __init__(
        self,
        x_label: _Optional[str] = ...,
        y_label: _Optional[str] = ...,
        curve: _Optional[_Union[Curve1D, _Mapping]] = ...,
    ) -> None: ...

class EfficiencyCurve(_message.Message):
    __slots__ = ("x_label", "y_label", "curve")
    X_LABEL_FIELD_NUMBER: _ClassVar[int]
    Y_LABEL_FIELD_NUMBER: _ClassVar[int]
    CURVE_FIELD_NUMBER: _ClassVar[int]
    x_label: str
    y_label: str
    curve: Curve1D
    def __init__(
        self,
        x_label: _Optional[str] = ...,
        y_label: _Optional[str] = ...,
        curve: _Optional[_Union[Curve1D, _Mapping]] = ...,
    ) -> None: ...

class BSFC(_message.Message):
    __slots__ = ("curve", "value")
    CURVE_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    curve: BSFCCurve
    value: float
    def __init__(
        self,
        curve: _Optional[_Union[BSFCCurve, _Mapping]] = ...,
        value: _Optional[float] = ...,
    ) -> None: ...

class Efficiency(_message.Message):
    __slots__ = ("curve", "value")
    CURVE_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    curve: EfficiencyCurve
    value: float
    def __init__(
        self,
        curve: _Optional[_Union[EfficiencyCurve, _Mapping]] = ...,
        value: _Optional[float] = ...,
    ) -> None: ...

class PowerCurve(_message.Message):
    __slots__ = ("x_label", "y_label", "curve")
    X_LABEL_FIELD_NUMBER: _ClassVar[int]
    Y_LABEL_FIELD_NUMBER: _ClassVar[int]
    CURVE_FIELD_NUMBER: _ClassVar[int]
    x_label: str
    y_label: str
    curve: Curve1D
    def __init__(
        self,
        x_label: _Optional[str] = ...,
        y_label: _Optional[str] = ...,
        curve: _Optional[_Union[Curve1D, _Mapping]] = ...,
    ) -> None: ...

class PropulsionPowerTimeSeries(_message.Message):
    __slots__ = ("x_label", "y_label", "propulsor_id", "curve")
    X_LABEL_FIELD_NUMBER: _ClassVar[int]
    Y_LABEL_FIELD_NUMBER: _ClassVar[int]
    PROPULSOR_ID_FIELD_NUMBER: _ClassVar[int]
    CURVE_FIELD_NUMBER: _ClassVar[int]
    x_label: str
    y_label: str
    propulsor_id: int
    curve: Curve1D
    def __init__(
        self,
        x_label: _Optional[str] = ...,
        y_label: _Optional[str] = ...,
        propulsor_id: _Optional[int] = ...,
        curve: _Optional[_Union[Curve1D, _Mapping]] = ...,
    ) -> None: ...

class AuxiliaryLoadTimeSeries(_message.Message):
    __slots__ = ("x_label", "y_label", "switchboard_id", "curve")
    X_LABEL_FIELD_NUMBER: _ClassVar[int]
    Y_LABEL_FIELD_NUMBER: _ClassVar[int]
    SWITCHBOARD_ID_FIELD_NUMBER: _ClassVar[int]
    CURVE_FIELD_NUMBER: _ClassVar[int]
    x_label: str
    y_label: str
    switchboard_id: int
    curve: Curve1D
    def __init__(
        self,
        x_label: _Optional[str] = ...,
        y_label: _Optional[str] = ...,
        switchboard_id: _Optional[int] = ...,
        curve: _Optional[_Union[Curve1D, _Mapping]] = ...,
    ) -> None: ...

class AuxiliaryLoad(_message.Message):
    __slots__ = ("switchboard_id", "load_kw")
    SWITCHBOARD_ID_FIELD_NUMBER: _ClassVar[int]
    LOAD_KW_FIELD_NUMBER: _ClassVar[int]
    switchboard_id: int
    load_kw: float
    def __init__(
        self, switchboard_id: _Optional[int] = ..., load_kw: _Optional[float] = ...
    ) -> None: ...

class EmissionCurve(_message.Message):
    __slots__ = ("x_label", "y_label", "curve", "emission_type")
    X_LABEL_FIELD_NUMBER: _ClassVar[int]
    Y_LABEL_FIELD_NUMBER: _ClassVar[int]
    CURVE_FIELD_NUMBER: _ClassVar[int]
    EMISSION_TYPE_FIELD_NUMBER: _ClassVar[int]
    x_label: str
    y_label: str
    curve: Curve1D
    emission_type: EmissionType
    def __init__(
        self,
        x_label: _Optional[str] = ...,
        y_label: _Optional[str] = ...,
        curve: _Optional[_Union[Curve1D, _Mapping]] = ...,
        emission_type: _Optional[_Union[EmissionType, str]] = ...,
    ) -> None: ...

class Gear(_message.Message):
    __slots__ = (
        "name",
        "gear_ratio",
        "rated_power_kw",
        "rated_speed_rpm",
        "efficiency",
        "order_from_switchboard_or_shaftline",
        "unit_price_usd",
    )
    NAME_FIELD_NUMBER: _ClassVar[int]
    GEAR_RATIO_FIELD_NUMBER: _ClassVar[int]
    RATED_POWER_KW_FIELD_NUMBER: _ClassVar[int]
    RATED_SPEED_RPM_FIELD_NUMBER: _ClassVar[int]
    EFFICIENCY_FIELD_NUMBER: _ClassVar[int]
    ORDER_FROM_SWITCHBOARD_OR_SHAFTLINE_FIELD_NUMBER: _ClassVar[int]
    UNIT_PRICE_USD_FIELD_NUMBER: _ClassVar[int]
    name: str
    gear_ratio: float
    rated_power_kw: float
    rated_speed_rpm: float
    efficiency: Efficiency
    order_from_switchboard_or_shaftline: int
    unit_price_usd: float
    def __init__(
        self,
        name: _Optional[str] = ...,
        gear_ratio: _Optional[float] = ...,
        rated_power_kw: _Optional[float] = ...,
        rated_speed_rpm: _Optional[float] = ...,
        efficiency: _Optional[_Union[Efficiency, _Mapping]] = ...,
        order_from_switchboard_or_shaftline: _Optional[int] = ...,
        unit_price_usd: _Optional[float] = ...,
    ) -> None: ...

class Fuel(_message.Message):
    __slots__ = ("fuel_type", "fuel_origin")
    FUEL_TYPE_FIELD_NUMBER: _ClassVar[int]
    FUEL_ORIGIN_FIELD_NUMBER: _ClassVar[int]
    fuel_type: FuelType
    fuel_origin: FuelOrigin
    def __init__(
        self,
        fuel_type: _Optional[_Union[FuelType, str]] = ...,
        fuel_origin: _Optional[_Union[FuelOrigin, str]] = ...,
    ) -> None: ...

class Engine(_message.Message):
    __slots__ = (
        "name",
        "rated_power_kw",
        "rated_speed_rpm",
        "bsfc",
        "main_fuel",
        "order_from_switchboard_or_shaftline",
        "pilot_bsfc",
        "pilot_fuel",
        "nox_calculation_method",
        "emission_curves",
        "engine_cycle_type",
        "unit_price_usd",
        "start_delay_s",
        "turn_off_power_kw",
    )

    class NOxCalculationMethod(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        TIER_2: _ClassVar[Engine.NOxCalculationMethod]
        TIER_1: _ClassVar[Engine.NOxCalculationMethod]
        TIER_3: _ClassVar[Engine.NOxCalculationMethod]
        CURVE: _ClassVar[Engine.NOxCalculationMethod]

    TIER_2: Engine.NOxCalculationMethod
    TIER_1: Engine.NOxCalculationMethod
    TIER_3: Engine.NOxCalculationMethod
    CURVE: Engine.NOxCalculationMethod

    class EngineCycleType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        NONE: _ClassVar[Engine.EngineCycleType]
        DIESEL: _ClassVar[Engine.EngineCycleType]
        OTTO: _ClassVar[Engine.EngineCycleType]
        LEAN_BURN_SPARK_IGNITION: _ClassVar[Engine.EngineCycleType]

    NONE: Engine.EngineCycleType
    DIESEL: Engine.EngineCycleType
    OTTO: Engine.EngineCycleType
    LEAN_BURN_SPARK_IGNITION: Engine.EngineCycleType
    NAME_FIELD_NUMBER: _ClassVar[int]
    RATED_POWER_KW_FIELD_NUMBER: _ClassVar[int]
    RATED_SPEED_RPM_FIELD_NUMBER: _ClassVar[int]
    BSFC_FIELD_NUMBER: _ClassVar[int]
    MAIN_FUEL_FIELD_NUMBER: _ClassVar[int]
    ORDER_FROM_SWITCHBOARD_OR_SHAFTLINE_FIELD_NUMBER: _ClassVar[int]
    PILOT_BSFC_FIELD_NUMBER: _ClassVar[int]
    PILOT_FUEL_FIELD_NUMBER: _ClassVar[int]
    NOX_CALCULATION_METHOD_FIELD_NUMBER: _ClassVar[int]
    EMISSION_CURVES_FIELD_NUMBER: _ClassVar[int]
    ENGINE_CYCLE_TYPE_FIELD_NUMBER: _ClassVar[int]
    UNIT_PRICE_USD_FIELD_NUMBER: _ClassVar[int]
    START_DELAY_S_FIELD_NUMBER: _ClassVar[int]
    TURN_OFF_POWER_KW_FIELD_NUMBER: _ClassVar[int]
    name: str
    rated_power_kw: float
    rated_speed_rpm: float
    bsfc: BSFC
    main_fuel: Fuel
    order_from_switchboard_or_shaftline: int
    pilot_bsfc: BSFC
    pilot_fuel: Fuel
    nox_calculation_method: Engine.NOxCalculationMethod
    emission_curves: _containers.RepeatedCompositeFieldContainer[EmissionCurve]
    engine_cycle_type: Engine.EngineCycleType
    unit_price_usd: float
    start_delay_s: float
    turn_off_power_kw: float
    def __init__(
        self,
        name: _Optional[str] = ...,
        rated_power_kw: _Optional[float] = ...,
        rated_speed_rpm: _Optional[float] = ...,
        bsfc: _Optional[_Union[BSFC, _Mapping]] = ...,
        main_fuel: _Optional[_Union[Fuel, _Mapping]] = ...,
        order_from_switchboard_or_shaftline: _Optional[int] = ...,
        pilot_bsfc: _Optional[_Union[BSFC, _Mapping]] = ...,
        pilot_fuel: _Optional[_Union[Fuel, _Mapping]] = ...,
        nox_calculation_method: _Optional[
            _Union[Engine.NOxCalculationMethod, str]
        ] = ...,
        emission_curves: _Optional[_Iterable[_Union[EmissionCurve, _Mapping]]] = ...,
        engine_cycle_type: _Optional[_Union[Engine.EngineCycleType, str]] = ...,
        unit_price_usd: _Optional[float] = ...,
        start_delay_s: _Optional[float] = ...,
        turn_off_power_kw: _Optional[float] = ...,
    ) -> None: ...

class COGAS(_message.Message):
    __slots__ = (
        "name",
        "rated_power_kw",
        "rated_speed_rpm",
        "efficiency",
        "gas_turbine_power_curve",
        "steam_turbine_power_curve",
        "fuel",
        "order_from_switchboard_or_shaftline",
        "nox_calculation_method",
        "emission_curves",
        "unit_price_usd",
        "start_delay_s",
        "turn_off_power_kw",
    )
    NAME_FIELD_NUMBER: _ClassVar[int]
    RATED_POWER_KW_FIELD_NUMBER: _ClassVar[int]
    RATED_SPEED_RPM_FIELD_NUMBER: _ClassVar[int]
    EFFICIENCY_FIELD_NUMBER: _ClassVar[int]
    GAS_TURBINE_POWER_CURVE_FIELD_NUMBER: _ClassVar[int]
    STEAM_TURBINE_POWER_CURVE_FIELD_NUMBER: _ClassVar[int]
    FUEL_FIELD_NUMBER: _ClassVar[int]
    ORDER_FROM_SWITCHBOARD_OR_SHAFTLINE_FIELD_NUMBER: _ClassVar[int]
    NOX_CALCULATION_METHOD_FIELD_NUMBER: _ClassVar[int]
    EMISSION_CURVES_FIELD_NUMBER: _ClassVar[int]
    UNIT_PRICE_USD_FIELD_NUMBER: _ClassVar[int]
    START_DELAY_S_FIELD_NUMBER: _ClassVar[int]
    TURN_OFF_POWER_KW_FIELD_NUMBER: _ClassVar[int]
    name: str
    rated_power_kw: float
    rated_speed_rpm: float
    efficiency: Efficiency
    gas_turbine_power_curve: PowerCurve
    steam_turbine_power_curve: PowerCurve
    fuel: Fuel
    order_from_switchboard_or_shaftline: int
    nox_calculation_method: Engine.NOxCalculationMethod
    emission_curves: _containers.RepeatedCompositeFieldContainer[EmissionCurve]
    unit_price_usd: float
    start_delay_s: float
    turn_off_power_kw: float
    def __init__(
        self,
        name: _Optional[str] = ...,
        rated_power_kw: _Optional[float] = ...,
        rated_speed_rpm: _Optional[float] = ...,
        efficiency: _Optional[_Union[Efficiency, _Mapping]] = ...,
        gas_turbine_power_curve: _Optional[_Union[PowerCurve, _Mapping]] = ...,
        steam_turbine_power_curve: _Optional[_Union[PowerCurve, _Mapping]] = ...,
        fuel: _Optional[_Union[Fuel, _Mapping]] = ...,
        order_from_switchboard_or_shaftline: _Optional[int] = ...,
        nox_calculation_method: _Optional[
            _Union[Engine.NOxCalculationMethod, str]
        ] = ...,
        emission_curves: _Optional[_Iterable[_Union[EmissionCurve, _Mapping]]] = ...,
        unit_price_usd: _Optional[float] = ...,
        start_delay_s: _Optional[float] = ...,
        turn_off_power_kw: _Optional[float] = ...,
    ) -> None: ...

class ElectricMachine(_message.Message):
    __slots__ = (
        "name",
        "rated_power_kw",
        "rated_speed_rpm",
        "efficiency",
        "order_from_switchboard_or_shaftline",
        "unit_price_usd",
    )
    NAME_FIELD_NUMBER: _ClassVar[int]
    RATED_POWER_KW_FIELD_NUMBER: _ClassVar[int]
    RATED_SPEED_RPM_FIELD_NUMBER: _ClassVar[int]
    EFFICIENCY_FIELD_NUMBER: _ClassVar[int]
    ORDER_FROM_SWITCHBOARD_OR_SHAFTLINE_FIELD_NUMBER: _ClassVar[int]
    UNIT_PRICE_USD_FIELD_NUMBER: _ClassVar[int]
    name: str
    rated_power_kw: float
    rated_speed_rpm: float
    efficiency: Efficiency
    order_from_switchboard_or_shaftline: int
    unit_price_usd: float
    def __init__(
        self,
        name: _Optional[str] = ...,
        rated_power_kw: _Optional[float] = ...,
        rated_speed_rpm: _Optional[float] = ...,
        efficiency: _Optional[_Union[Efficiency, _Mapping]] = ...,
        order_from_switchboard_or_shaftline: _Optional[int] = ...,
        unit_price_usd: _Optional[float] = ...,
    ) -> None: ...

class Battery(_message.Message):
    __slots__ = (
        "name",
        "energy_capacity_kwh",
        "rated_charging_rate_c",
        "rated_discharging_rate_c",
        "efficiency_charging",
        "efficiency_discharging",
        "initial_state_of_charge",
        "order_from_switchboard_or_shaftline",
        "unit_price_usd",
        "self_discharge_percent_per_day",
        "state_of_energy_minimum",
        "state_of_energy_maximum",
    )
    NAME_FIELD_NUMBER: _ClassVar[int]
    ENERGY_CAPACITY_KWH_FIELD_NUMBER: _ClassVar[int]
    RATED_CHARGING_RATE_C_FIELD_NUMBER: _ClassVar[int]
    RATED_DISCHARGING_RATE_C_FIELD_NUMBER: _ClassVar[int]
    EFFICIENCY_CHARGING_FIELD_NUMBER: _ClassVar[int]
    EFFICIENCY_DISCHARGING_FIELD_NUMBER: _ClassVar[int]
    INITIAL_STATE_OF_CHARGE_FIELD_NUMBER: _ClassVar[int]
    ORDER_FROM_SWITCHBOARD_OR_SHAFTLINE_FIELD_NUMBER: _ClassVar[int]
    UNIT_PRICE_USD_FIELD_NUMBER: _ClassVar[int]
    SELF_DISCHARGE_PERCENT_PER_DAY_FIELD_NUMBER: _ClassVar[int]
    STATE_OF_ENERGY_MINIMUM_FIELD_NUMBER: _ClassVar[int]
    STATE_OF_ENERGY_MAXIMUM_FIELD_NUMBER: _ClassVar[int]
    name: str
    energy_capacity_kwh: float
    rated_charging_rate_c: float
    rated_discharging_rate_c: float
    efficiency_charging: float
    efficiency_discharging: float
    initial_state_of_charge: float
    order_from_switchboard_or_shaftline: int
    unit_price_usd: float
    self_discharge_percent_per_day: float
    state_of_energy_minimum: float
    state_of_energy_maximum: float
    def __init__(
        self,
        name: _Optional[str] = ...,
        energy_capacity_kwh: _Optional[float] = ...,
        rated_charging_rate_c: _Optional[float] = ...,
        rated_discharging_rate_c: _Optional[float] = ...,
        efficiency_charging: _Optional[float] = ...,
        efficiency_discharging: _Optional[float] = ...,
        initial_state_of_charge: _Optional[float] = ...,
        order_from_switchboard_or_shaftline: _Optional[int] = ...,
        unit_price_usd: _Optional[float] = ...,
        self_discharge_percent_per_day: _Optional[float] = ...,
        state_of_energy_minimum: _Optional[float] = ...,
        state_of_energy_maximum: _Optional[float] = ...,
    ) -> None: ...

class ElectricComponent(_message.Message):
    __slots__ = (
        "name",
        "rated_power_kw",
        "efficiency",
        "order_from_switchboard_or_shaftline",
        "unit_price_usd",
    )
    NAME_FIELD_NUMBER: _ClassVar[int]
    RATED_POWER_KW_FIELD_NUMBER: _ClassVar[int]
    EFFICIENCY_FIELD_NUMBER: _ClassVar[int]
    ORDER_FROM_SWITCHBOARD_OR_SHAFTLINE_FIELD_NUMBER: _ClassVar[int]
    UNIT_PRICE_USD_FIELD_NUMBER: _ClassVar[int]
    name: str
    rated_power_kw: float
    efficiency: Efficiency
    order_from_switchboard_or_shaftline: int
    unit_price_usd: float
    def __init__(
        self,
        name: _Optional[str] = ...,
        rated_power_kw: _Optional[float] = ...,
        efficiency: _Optional[_Union[Efficiency, _Mapping]] = ...,
        order_from_switchboard_or_shaftline: _Optional[int] = ...,
        unit_price_usd: _Optional[float] = ...,
    ) -> None: ...

class FuelCell(_message.Message):
    __slots__ = (
        "name",
        "rated_power_kw",
        "efficiency",
        "order_from_switchboard_or_shaftline",
        "fuel",
        "unit_price_usd",
        "number_modules",
        "power_minimum_specific",
        "start_delay_s",
    )
    NAME_FIELD_NUMBER: _ClassVar[int]
    RATED_POWER_KW_FIELD_NUMBER: _ClassVar[int]
    EFFICIENCY_FIELD_NUMBER: _ClassVar[int]
    ORDER_FROM_SWITCHBOARD_OR_SHAFTLINE_FIELD_NUMBER: _ClassVar[int]
    FUEL_FIELD_NUMBER: _ClassVar[int]
    UNIT_PRICE_USD_FIELD_NUMBER: _ClassVar[int]
    NUMBER_MODULES_FIELD_NUMBER: _ClassVar[int]
    POWER_MINIMUM_SPECIFIC_FIELD_NUMBER: _ClassVar[int]
    START_DELAY_S_FIELD_NUMBER: _ClassVar[int]
    name: str
    rated_power_kw: float
    efficiency: Efficiency
    order_from_switchboard_or_shaftline: int
    fuel: Fuel
    unit_price_usd: float
    number_modules: int
    power_minimum_specific: float
    start_delay_s: float
    def __init__(
        self,
        name: _Optional[str] = ...,
        rated_power_kw: _Optional[float] = ...,
        efficiency: _Optional[_Union[Efficiency, _Mapping]] = ...,
        order_from_switchboard_or_shaftline: _Optional[int] = ...,
        fuel: _Optional[_Union[Fuel, _Mapping]] = ...,
        unit_price_usd: _Optional[float] = ...,
        number_modules: _Optional[int] = ...,
        power_minimum_specific: _Optional[float] = ...,
        start_delay_s: _Optional[float] = ...,
    ) -> None: ...

class Propeller(_message.Message):
    __slots__ = (
        "name",
        "efficiency",
        "propulsor_id",
        "order_from_switchboard_or_shaftline",
    )
    NAME_FIELD_NUMBER: _ClassVar[int]
    EFFICIENCY_FIELD_NUMBER: _ClassVar[int]
    PROPULSOR_ID_FIELD_NUMBER: _ClassVar[int]
    ORDER_FROM_SWITCHBOARD_OR_SHAFTLINE_FIELD_NUMBER: _ClassVar[int]
    name: str
    efficiency: Efficiency
    propulsor_id: int
    order_from_switchboard_or_shaftline: int
    def __init__(
        self,
        name: _Optional[str] = ...,
        efficiency: _Optional[_Union[Efficiency, _Mapping]] = ...,
        propulsor_id: _Optional[int] = ...,
        order_from_switchboard_or_shaftline: _Optional[int] = ...,
    ) -> None: ...

class BusBreaker(_message.Message):
    __slots__ = ("switchboard_to",)
    SWITCHBOARD_TO_FIELD_NUMBER: _ClassVar[int]
    switchboard_to: int
    def __init__(self, switchboard_to: _Optional[int] = ...) -> None: ...

class SuperCapacitor(_message.Message):
    __slots__ = (
        "name",
        "energy_capacity_wh",
        "rated_power_kw",
        "efficiency_charging",
        "efficiency_discharging",
        "initial_state_of_charge",
        "order_from_switchboard_or_shaftline",
        "unit_price_usd",
    )
    NAME_FIELD_NUMBER: _ClassVar[int]
    ENERGY_CAPACITY_WH_FIELD_NUMBER: _ClassVar[int]
    RATED_POWER_KW_FIELD_NUMBER: _ClassVar[int]
    EFFICIENCY_CHARGING_FIELD_NUMBER: _ClassVar[int]
    EFFICIENCY_DISCHARGING_FIELD_NUMBER: _ClassVar[int]
    INITIAL_STATE_OF_CHARGE_FIELD_NUMBER: _ClassVar[int]
    ORDER_FROM_SWITCHBOARD_OR_SHAFTLINE_FIELD_NUMBER: _ClassVar[int]
    UNIT_PRICE_USD_FIELD_NUMBER: _ClassVar[int]
    name: str
    energy_capacity_wh: float
    rated_power_kw: float
    efficiency_charging: float
    efficiency_discharging: float
    initial_state_of_charge: float
    order_from_switchboard_or_shaftline: int
    unit_price_usd: float
    def __init__(
        self,
        name: _Optional[str] = ...,
        energy_capacity_wh: _Optional[float] = ...,
        rated_power_kw: _Optional[float] = ...,
        efficiency_charging: _Optional[float] = ...,
        efficiency_discharging: _Optional[float] = ...,
        initial_state_of_charge: _Optional[float] = ...,
        order_from_switchboard_or_shaftline: _Optional[int] = ...,
        unit_price_usd: _Optional[float] = ...,
    ) -> None: ...

class MechanicalComponent(_message.Message):
    __slots__ = (
        "name",
        "rated_power_kw",
        "efficiency",
        "order_from_switchboard_or_shaftline",
        "unit_price_usd",
    )
    NAME_FIELD_NUMBER: _ClassVar[int]
    RATED_POWER_KW_FIELD_NUMBER: _ClassVar[int]
    EFFICIENCY_FIELD_NUMBER: _ClassVar[int]
    ORDER_FROM_SWITCHBOARD_OR_SHAFTLINE_FIELD_NUMBER: _ClassVar[int]
    UNIT_PRICE_USD_FIELD_NUMBER: _ClassVar[int]
    name: str
    rated_power_kw: float
    efficiency: Efficiency
    order_from_switchboard_or_shaftline: int
    unit_price_usd: float
    def __init__(
        self,
        name: _Optional[str] = ...,
        rated_power_kw: _Optional[float] = ...,
        efficiency: _Optional[_Union[Efficiency, _Mapping]] = ...,
        order_from_switchboard_or_shaftline: _Optional[int] = ...,
        unit_price_usd: _Optional[float] = ...,
    ) -> None: ...

class Subsystem(_message.Message):
    __slots__ = (
        "gear",
        "engine",
        "electric_machine",
        "transformer",
        "converter1",
        "converter2",
        "battery",
        "fuel_cell",
        "propeller",
        "bus_breaker",
        "supercapacitor",
        "other_load",
        "cogas",
        "power_type",
        "component_type",
        "name",
        "rated_power_kw",
        "rated_speed_rpm",
        "ramp_up_rate_limit_percent_per_second",
        "ramp_down_rate_limit_percent_per_second",
        "base_load_order",
    )

    class PowerType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        NONE1: _ClassVar[Subsystem.PowerType]
        POWER_SOURCE: _ClassVar[Subsystem.PowerType]
        POWER_CONSUMER: _ClassVar[Subsystem.PowerType]
        PTI_PTO: _ClassVar[Subsystem.PowerType]
        ENERGY_STORAGE: _ClassVar[Subsystem.PowerType]
        SHORE_CONNECTION: _ClassVar[Subsystem.PowerType]

    NONE1: Subsystem.PowerType
    POWER_SOURCE: Subsystem.PowerType
    POWER_CONSUMER: Subsystem.PowerType
    PTI_PTO: Subsystem.PowerType
    ENERGY_STORAGE: Subsystem.PowerType
    SHORE_CONNECTION: Subsystem.PowerType

    class ComponentType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        NONE: _ClassVar[Subsystem.ComponentType]
        MAIN_ENGINE: _ClassVar[Subsystem.ComponentType]
        AUXILIARY_ENGINE: _ClassVar[Subsystem.ComponentType]
        GENERATOR: _ClassVar[Subsystem.ComponentType]
        PROPULSION_DRIVE: _ClassVar[Subsystem.ComponentType]
        OTHER_LOAD: _ClassVar[Subsystem.ComponentType]
        PTI_PTO_SYSTEM: _ClassVar[Subsystem.ComponentType]
        BATTERY_SYSTEM: _ClassVar[Subsystem.ComponentType]
        FUEL_CELL_SYSTEM: _ClassVar[Subsystem.ComponentType]
        RECTIFIER: _ClassVar[Subsystem.ComponentType]
        MAIN_ENGINE_WITH_GEARBOX: _ClassVar[Subsystem.ComponentType]
        ELECTRIC_MOTOR: _ClassVar[Subsystem.ComponentType]
        GENSET: _ClassVar[Subsystem.ComponentType]
        TRANSFORMER: _ClassVar[Subsystem.ComponentType]
        INVERTER: _ClassVar[Subsystem.ComponentType]
        CIRCUIT_BREAKER: _ClassVar[Subsystem.ComponentType]
        ACTIVE_FRONT_END: _ClassVar[Subsystem.ComponentType]
        POWER_CONVERTER: _ClassVar[Subsystem.ComponentType]
        SYNCHRONOUS_MACHINE: _ClassVar[Subsystem.ComponentType]
        INDUCTION_MACHINE: _ClassVar[Subsystem.ComponentType]
        GEARBOX: _ClassVar[Subsystem.ComponentType]
        FUEL_CELL: _ClassVar[Subsystem.ComponentType]
        PROPELLER_LOAD: _ClassVar[Subsystem.ComponentType]
        OTHER_MECHANICAL_LOAD: _ClassVar[Subsystem.ComponentType]
        BATTERY: _ClassVar[Subsystem.ComponentType]
        SUPERCAPACITOR: _ClassVar[Subsystem.ComponentType]
        SUPERCAPACITOR_SYSTEM: _ClassVar[Subsystem.ComponentType]
        SHORE_POWER: _ClassVar[Subsystem.ComponentType]
        COGAS: _ClassVar[Subsystem.ComponentType]
        COGES: _ClassVar[Subsystem.ComponentType]

    NONE: Subsystem.ComponentType
    MAIN_ENGINE: Subsystem.ComponentType
    AUXILIARY_ENGINE: Subsystem.ComponentType
    GENERATOR: Subsystem.ComponentType
    PROPULSION_DRIVE: Subsystem.ComponentType
    OTHER_LOAD: Subsystem.ComponentType
    PTI_PTO_SYSTEM: Subsystem.ComponentType
    BATTERY_SYSTEM: Subsystem.ComponentType
    FUEL_CELL_SYSTEM: Subsystem.ComponentType
    RECTIFIER: Subsystem.ComponentType
    MAIN_ENGINE_WITH_GEARBOX: Subsystem.ComponentType
    ELECTRIC_MOTOR: Subsystem.ComponentType
    GENSET: Subsystem.ComponentType
    TRANSFORMER: Subsystem.ComponentType
    INVERTER: Subsystem.ComponentType
    CIRCUIT_BREAKER: Subsystem.ComponentType
    ACTIVE_FRONT_END: Subsystem.ComponentType
    POWER_CONVERTER: Subsystem.ComponentType
    SYNCHRONOUS_MACHINE: Subsystem.ComponentType
    INDUCTION_MACHINE: Subsystem.ComponentType
    GEARBOX: Subsystem.ComponentType
    FUEL_CELL: Subsystem.ComponentType
    PROPELLER_LOAD: Subsystem.ComponentType
    OTHER_MECHANICAL_LOAD: Subsystem.ComponentType
    BATTERY: Subsystem.ComponentType
    SUPERCAPACITOR: Subsystem.ComponentType
    SUPERCAPACITOR_SYSTEM: Subsystem.ComponentType
    SHORE_POWER: Subsystem.ComponentType
    COGAS: Subsystem.ComponentType
    COGES: Subsystem.ComponentType
    GEAR_FIELD_NUMBER: _ClassVar[int]
    ENGINE_FIELD_NUMBER: _ClassVar[int]
    ELECTRIC_MACHINE_FIELD_NUMBER: _ClassVar[int]
    TRANSFORMER_FIELD_NUMBER: _ClassVar[int]
    CONVERTER1_FIELD_NUMBER: _ClassVar[int]
    CONVERTER2_FIELD_NUMBER: _ClassVar[int]
    BATTERY_FIELD_NUMBER: _ClassVar[int]
    FUEL_CELL_FIELD_NUMBER: _ClassVar[int]
    PROPELLER_FIELD_NUMBER: _ClassVar[int]
    BUS_BREAKER_FIELD_NUMBER: _ClassVar[int]
    SUPERCAPACITOR_FIELD_NUMBER: _ClassVar[int]
    OTHER_LOAD_FIELD_NUMBER: _ClassVar[int]
    COGAS_FIELD_NUMBER: _ClassVar[int]
    POWER_TYPE_FIELD_NUMBER: _ClassVar[int]
    COMPONENT_TYPE_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    RATED_POWER_KW_FIELD_NUMBER: _ClassVar[int]
    RATED_SPEED_RPM_FIELD_NUMBER: _ClassVar[int]
    RAMP_UP_RATE_LIMIT_PERCENT_PER_SECOND_FIELD_NUMBER: _ClassVar[int]
    RAMP_DOWN_RATE_LIMIT_PERCENT_PER_SECOND_FIELD_NUMBER: _ClassVar[int]
    BASE_LOAD_ORDER_FIELD_NUMBER: _ClassVar[int]
    gear: Gear
    engine: Engine
    electric_machine: ElectricMachine
    transformer: ElectricComponent
    converter1: ElectricComponent
    converter2: ElectricComponent
    battery: Battery
    fuel_cell: FuelCell
    propeller: Propeller
    bus_breaker: BusBreaker
    supercapacitor: SuperCapacitor
    other_load: ElectricComponent
    cogas: COGAS
    power_type: Subsystem.PowerType
    component_type: Subsystem.ComponentType
    name: str
    rated_power_kw: float
    rated_speed_rpm: float
    ramp_up_rate_limit_percent_per_second: float
    ramp_down_rate_limit_percent_per_second: float
    base_load_order: int
    def __init__(
        self,
        gear: _Optional[_Union[Gear, _Mapping]] = ...,
        engine: _Optional[_Union[Engine, _Mapping]] = ...,
        electric_machine: _Optional[_Union[ElectricMachine, _Mapping]] = ...,
        transformer: _Optional[_Union[ElectricComponent, _Mapping]] = ...,
        converter1: _Optional[_Union[ElectricComponent, _Mapping]] = ...,
        converter2: _Optional[_Union[ElectricComponent, _Mapping]] = ...,
        battery: _Optional[_Union[Battery, _Mapping]] = ...,
        fuel_cell: _Optional[_Union[FuelCell, _Mapping]] = ...,
        propeller: _Optional[_Union[Propeller, _Mapping]] = ...,
        bus_breaker: _Optional[_Union[BusBreaker, _Mapping]] = ...,
        supercapacitor: _Optional[_Union[SuperCapacitor, _Mapping]] = ...,
        other_load: _Optional[_Union[ElectricComponent, _Mapping]] = ...,
        cogas: _Optional[_Union[COGAS, _Mapping]] = ...,
        power_type: _Optional[_Union[Subsystem.PowerType, str]] = ...,
        component_type: _Optional[_Union[Subsystem.ComponentType, str]] = ...,
        name: _Optional[str] = ...,
        rated_power_kw: _Optional[float] = ...,
        rated_speed_rpm: _Optional[float] = ...,
        ramp_up_rate_limit_percent_per_second: _Optional[float] = ...,
        ramp_down_rate_limit_percent_per_second: _Optional[float] = ...,
        base_load_order: _Optional[int] = ...,
    ) -> None: ...

class Switchboard(_message.Message):
    __slots__ = ("switchboard_id", "subsystems")
    SWITCHBOARD_ID_FIELD_NUMBER: _ClassVar[int]
    SUBSYSTEMS_FIELD_NUMBER: _ClassVar[int]
    switchboard_id: int
    subsystems: _containers.RepeatedCompositeFieldContainer[Subsystem]
    def __init__(
        self,
        switchboard_id: _Optional[int] = ...,
        subsystems: _Optional[_Iterable[_Union[Subsystem, _Mapping]]] = ...,
    ) -> None: ...

class ShaftLine(_message.Message):
    __slots__ = ("shaft_line_id", "subsystems")
    SHAFT_LINE_ID_FIELD_NUMBER: _ClassVar[int]
    SUBSYSTEMS_FIELD_NUMBER: _ClassVar[int]
    shaft_line_id: int
    subsystems: _containers.RepeatedCompositeFieldContainer[Subsystem]
    def __init__(
        self,
        shaft_line_id: _Optional[int] = ...,
        subsystems: _Optional[_Iterable[_Union[Subsystem, _Mapping]]] = ...,
    ) -> None: ...

class MechanicalSystem(_message.Message):
    __slots__ = ("shaft_lines",)
    SHAFT_LINES_FIELD_NUMBER: _ClassVar[int]
    shaft_lines: _containers.RepeatedCompositeFieldContainer[ShaftLine]
    def __init__(
        self, shaft_lines: _Optional[_Iterable[_Union[ShaftLine, _Mapping]]] = ...
    ) -> None: ...

class ElectricSystem(_message.Message):
    __slots__ = ("switchboards",)
    SWITCHBOARDS_FIELD_NUMBER: _ClassVar[int]
    switchboards: _containers.RepeatedCompositeFieldContainer[Switchboard]
    def __init__(
        self, switchboards: _Optional[_Iterable[_Union[Switchboard, _Mapping]]] = ...
    ) -> None: ...

class FuelStorage(_message.Message):
    __slots__ = ("fuel_type", "capacity_kg")
    FUEL_TYPE_FIELD_NUMBER: _ClassVar[int]
    CAPACITY_KG_FIELD_NUMBER: _ClassVar[int]
    fuel_type: FuelType
    capacity_kg: float
    def __init__(
        self,
        fuel_type: _Optional[_Union[FuelType, str]] = ...,
        capacity_kg: _Optional[float] = ...,
    ) -> None: ...

class MachinerySystem(_message.Message):
    __slots__ = (
        "name",
        "propulsion_type",
        "fuel_storage",
        "maximum_allowed_genset_load_percentage",
        "mechanical_system",
        "electric_system",
        "maximum_allowed_fuel_cell_load_percentage",
        "average_base_load_percentage",
    )

    class PropulsionType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        MECHANICAL: _ClassVar[MachinerySystem.PropulsionType]
        ELECTRIC: _ClassVar[MachinerySystem.PropulsionType]
        HYBRID: _ClassVar[MachinerySystem.PropulsionType]

    MECHANICAL: MachinerySystem.PropulsionType
    ELECTRIC: MachinerySystem.PropulsionType
    HYBRID: MachinerySystem.PropulsionType
    NAME_FIELD_NUMBER: _ClassVar[int]
    PROPULSION_TYPE_FIELD_NUMBER: _ClassVar[int]
    FUEL_STORAGE_FIELD_NUMBER: _ClassVar[int]
    MAXIMUM_ALLOWED_GENSET_LOAD_PERCENTAGE_FIELD_NUMBER: _ClassVar[int]
    MECHANICAL_SYSTEM_FIELD_NUMBER: _ClassVar[int]
    ELECTRIC_SYSTEM_FIELD_NUMBER: _ClassVar[int]
    MAXIMUM_ALLOWED_FUEL_CELL_LOAD_PERCENTAGE_FIELD_NUMBER: _ClassVar[int]
    AVERAGE_BASE_LOAD_PERCENTAGE_FIELD_NUMBER: _ClassVar[int]
    name: str
    propulsion_type: MachinerySystem.PropulsionType
    fuel_storage: _containers.RepeatedCompositeFieldContainer[FuelStorage]
    maximum_allowed_genset_load_percentage: float
    mechanical_system: MechanicalSystem
    electric_system: ElectricSystem
    maximum_allowed_fuel_cell_load_percentage: float
    average_base_load_percentage: float
    def __init__(
        self,
        name: _Optional[str] = ...,
        propulsion_type: _Optional[_Union[MachinerySystem.PropulsionType, str]] = ...,
        fuel_storage: _Optional[_Iterable[_Union[FuelStorage, _Mapping]]] = ...,
        maximum_allowed_genset_load_percentage: _Optional[float] = ...,
        mechanical_system: _Optional[_Union[MechanicalSystem, _Mapping]] = ...,
        electric_system: _Optional[_Union[ElectricSystem, _Mapping]] = ...,
        maximum_allowed_fuel_cell_load_percentage: _Optional[float] = ...,
        average_base_load_percentage: _Optional[float] = ...,
    ) -> None: ...
